import unittest
from wicd import wnettools

class TestWnettools(unittest.TestCase):
	def setUp(self):
		self.interface = wnettools.BaseInterface('eth0')
	
	def test_find_wireless_interface(self):
		interfaces = wnettools.GetWirelessInterfaces()
		# wlan0 may change depending on your system
		self.assertTrue('wlan0' in interfaces)
		
	def test_find_wired_interface(self):
		interfaces = wnettools.GetWiredInterfaces()
		# eth0 may change depending on your system
		self.assertTrue('eth0' in interfaces)
		
	def test_wext_is_valid_wpasupplicant_driver(self):
		self.assertTrue(wnettools.IsValidWpaSuppDriver('wext'))
		
	def test_needs_external_calls_not_implemented(self):
		self.assertRaises(NotImplementedError, wnettools.NeedsExternalCalls)
		
	def test_get_ip_not_implemented(self):
		self.assertRaises(NotImplementedError, self.interface.GetIP)
		
	def test_is_up_not_implemented(self):
		self.assertRaises(NotImplementedError, self.interface.IsUp)
		
	def test_enable_debug_mode(self):
		self.interface.SetDebugMode(True)
		self.assertTrue(self.interface.verbose)
		
	def test_disable_debug_mode(self):
		self.interface.SetDebugMode(False)
		self.assertFalse(self.interface.verbose)
		
	def test_interface_name_sanitation(self):
		interface = wnettools.BaseInterface('blahblah; uptime > /tmp/blah | cat')
		self.assertEquals(interface.iface, 'blahblahuptimetmpblahcat')
		
	def test_freq_translation_low(self):
		freq = '2.412 GHz'
		interface = wnettools.BaseWirelessInterface('wlan0')
		self.assertEquals(interface._FreqToChannel(freq), 1)
		
	def test_freq_translation_high(self):
		freq = '2.484 GHz'
		interface = wnettools.BaseWirelessInterface('wlan0')
		self.assertEquals(interface._FreqToChannel(freq), 14)
		
	def test_generate_psk(self):
		interface = wnettools.BaseWirelessInterface('wlan0')
		psk = interface.GeneratePSK({'essid' : 'Network 1', 'key' : 'arandompassphrase'})
		self.assertEquals(psk, 'd70463014514f4b4ebb8e3aebbdec13f4437ac3a9af084b3433f3710e658a7be')

def suite():
	suite = unittest.TestSuite()
	tests = []
	[ tests.append(test) for test in dir(TestWnettools) if test.startswith('test') ]
	for test in tests:
		suite.addTest(TestWnettools(test))
	return suite

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" ioctl Network interface control tools for wicd.

This module implements functions to control and obtain information from
network interfaces.  It utilizes ioctl calls and python modules to
obtain this information whenever possible.

class Interface() -- Control a network interface.
class WiredInterface() -- Control a wired network interface.
class WirelessInterface() -- Control a wireless network interface.

"""

#
#   Copyright (C) 2008-2009 Adam Blackburn
#   Copyright (C) 2008-2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from wicd import misc
from wicd import wpath
from wicd.wnettools import GetDefaultGateway, GetWiredInterfaces, \
GetWirelessInterfaces, IsValidWpaSuppDriver, BaseWirelessInterface, \
BaseWiredInterface, BaseInterface, GetWpaSupplicantDrivers, wep_pattern, \
signaldbm_pattern, neediface

try:
    import iwscan
    IWSCAN_AVAIL = True
except ImportError:
    print "WARNING: python-iwscan not found, falling back to using iwlist scan."
    IWSCAN_AVAIL = False
try:
    import wpactrl
    WPACTRL_AVAIL = True
except ImportError:
    print "WARNING: python-wpactrl not found, falling back to using wpa_cli."
    WPACTRL_AVAIL = False

import re
import os
import time
import socket
import fcntl
import struct
import array


NAME = "ioctl"
UPDATE_INTERVAL = 4
DESCRIPTION = """IOCTL (experimental) backend

This backend uses IOCTL calls and python libraries to query
network information whenever possible.  This makes it fast,
but it may not work properly on all systems.

(Optional) Dependencies:
python-wpactrl (http://projects.otaku42.de/wiki/PythonWpaCtrl)
python-iwscan (http://projects.otaku42.de/browser/python-iwscan/)"""

RALINK_DRIVER = 'ralink legacy'
NONE_DRIVER = 'none'

# Got these from /usr/include/linux/wireless.h
SIOCGIWESSID = 0x8B1B
SIOCGIWRANGE = 0x8B0B
SIOCGIWAP = 0x8B15
SIOCGIWSTATS = 0x8B0F
SIOCGIWRATE = 0x8B21

# Got these from /usr/include/sockios.h
SIOCGIFADDR = 0x8915
SIOCGIFHWADDR = 0x8927
SIOCGMIIPHY = 0x8947
SIOCETHTOOL = 0x8946
SIOCGIFFLAGS = 0x8913


def get_iw_ioctl_result(iface, call):
    """ Makes the given ioctl call and returns the results.

    Keyword arguments:
    call -- The ioctl call to make

    Returns:
    The results of the ioctl call.

    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    buff = array.array('c', '\0' * 32)
    addr, length = buff.buffer_info()
    arg = struct.pack('Pi', addr, length)
    data = (iface + '\0' * 16)[:16] + arg
    try:
        result = fcntl.ioctl(s.fileno(), call, data)
    except IOError:
        return None
    except OSError:
        return None
    return buff.tostring()

def NeedsExternalCalls(*args, **kargs):
    """ Return False, since this backend doesn't use any external apps. """
    return False


class Interface(BaseInterface):
    """ Control a network interface. """
    def __init__(self, iface, verbose=False):
        """ Initialise the object.

        Keyword arguments:
        iface -- the name of the interface
        verbose -- whether to print every command run

        """
        BaseInterface.__init__(self, iface, verbose)
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.Check()

    def CheckWirelessTools(self):
        """ Check for the existence needed wireless tools """
        if not WPACTRL_AVAIL:
            BaseInterface.CheckWirelessTools(self)

    @neediface("")
    def GetIP(self, ifconfig=""):
        """ Get the IP address of the interface.

        Returns:
        The IP address of the interface in dotted quad form.

        """
        ifstruct = struct.pack('256s', self.iface)
        try:
            raw_ip = fcntl.ioctl(self.sock.fileno(), SIOCGIFADDR, ifstruct)
        except IOError:
            return None
        except OSError:
            return None

        return socket.inet_ntoa(raw_ip[20:24])

    @neediface(False)
    def IsUp(self):
        """ Determines if the interface is up.

        Returns:
        True if the interface is up, False otherwise.

        """
        data = (self.iface + '\0' * 16)[:18]
        try:
            result = fcntl.ioctl(self.sock.fileno(), SIOCGIFFLAGS, data)
        except IOError, e:
            if self.verbose:
                print "SIOCGIFFLAGS failed: " + str(e)
            return False

        flags, = struct.unpack('H', result[16:18])
        return bool(flags & 1)


class WiredInterface(Interface, BaseWiredInterface):
    """ Control a wired network interface. """
    def __init__(self, iface, verbose=False):
        """ Initialise the wired network interface class.

        Keyword arguments:
        iface -- name of the interface
        verbose -- print all commands

        """
        BaseWiredInterface.__init__(self, iface, verbose)
        Interface.__init__(self, iface, verbose)

    @neediface(False)
    def GetPluggedIn(self):
        """ Get the current physical connection state.

        The method will first attempt to use ethtool do determine
        physical connection state.  Should ethtool fail to run properly,
        mii-tool will be used instead.

        Returns:
        True if a link is detected, False otherwise.

        """
        if self.ethtool_cmd and self.link_detect in [misc.ETHTOOL, misc.AUTO]:
            return self._eth_get_plugged_in()
        elif self.miitool_cmd and self.link_detect in [misc.MIITOOL, misc.AUTO]:
            return self._mii_get_plugged_in()
        else:
            print ('Error: No way of checking for a wired connection. Make' +
                   'sure that either mii-tool or ethtool is installed.')
            return False

    def _eth_get_plugged_in(self):
        """ Use ethtool to determine the physical connection state.

        Returns:
        True if a link is detected, False otherwise.

        """
        if not self.IsUp():
            self.Up()
            time.sleep(5)
        buff = array.array('i', [0x0000000a, 0x00000000])
        addr, length = buff.buffer_info()
        arg = struct.pack('Pi', addr, length)
        data = (self.iface + '\0' * 16)[:16] + arg
        try:
            fcntl.ioctl(self.sock.fileno(), SIOCETHTOOL, data)
        except IOError, e:
            if self.verbose:
                print 'SIOCETHTOOL failed: ' + str(e)
            return False
        return bool(buff.tolist()[1])

    def _mii_get_plugged_in(self):
        """ Use mii-tool to determine the physical connection state. 

        Returns:
        True if a link is detected, False otherwise.

        """
        if not self.IsUp():
            self.Up()
            time.sleep(2.5)
        buff = struct.pack('16shhhh', (self.iface + '\0' * 16)[:16], 0, 1,
                           0x0004, 0)
        try:
            result = fcntl.ioctl(self.sock.fileno(), SIOCGMIIPHY, buff)
        except IOError, e:
            if self.verbose:
                print 'SIOCGMIIPHY failed: ' + str(e)
            return False
        reg = struct.unpack('16shhhh', result)[-1]
        return bool(reg & 0x0004)


class WirelessInterface(Interface, BaseWirelessInterface):
    """ Control a wireless network interface. """
    def __init__(self, iface, verbose=False, wpa_driver='wext'):
        """ Initialise the wireless network interface class.

        Keyword arguments:
        iface -- name of the interface
        verbose -- print all commands

        """
        BaseWirelessInterface.__init__(self, iface, verbose,
                                                 wpa_driver)
        Interface.__init__(self, iface, verbose)
        self.scan_iface = None
        self.CheckWirelessTools()

    @neediface([])
    def GetNetworks(self):
        """ Get a list of available wireless networks.

        Returns:
        A list containing available wireless networks.

        """
        if not IWSCAN_AVAIL:
            # Use the slow version if python-iwscan isn't available.
            return BaseWirelessInterface.GetNetworks(self)
        
        if not self.scan_iface:
            try:
                self.scan_iface = iwscan.WirelessInterface(self.iface)
            except iwscan.error, e:
                print "GetNetworks caught an exception: %s" % e
                return []

        try:
            results = self.scan_iface.Scan()
        except iwscan.error, e:
            print "ERROR: %s" % e
            return []
        return filter(None, [self._parse_ap(cell) for cell in results])

    def _parse_ap(self, cell):
        """ Parse a single cell from the python-iwscan list. """
        ap = {}
        try:
            ap['essid'] = misc.to_unicode(cell['essid'])
        except UnicodeError:
            print 'Unicode problem with the current network essid, ignoring!!'
            return None

        if ap['essid'] in [ "", '<hidden>']:
            ap['essid'] = '<hidden>'
            ap['hidden'] = True
        else:
            ap['hidden'] = False

        if cell["channel"]:
            ap["channel"] = cell["channel"]
        else:
            ap["channel"] = self._FreqToChannel(cell["frequency"])

        ap["bssid"] = cell["bssid"]
        ap["mode"] = cell["mode"]
        ap["bitrates"] = cell["bitrate"]

        if cell["enc"]:
            ap["encryption"] = True
            if cell["ie"] and cell["ie"].get('type'):
                if "WPA2" in cell['ie']['type'].upper():
                    ap['encryption_method'] = 'WPA2'
                elif "WPA" in cell['ie']['type'].upper():
                    ap['encryption_method'] = 'WPA'
            else:
                ap['encryption_method'] = 'WEP'
        else:
            ap["encryption"] = False

        # Link Quality
        ap['qual_found'] = True
        ap['quality'] = self._get_link_quality(cell['stats'])
        if ap['quality'] is None:
            ap['qual_found'] = False
            ap['quality'] = -1

        # Signal Strength (only used if user doesn't want link
        # quality displayed or it isn't found)
        if misc.RunRegex(signaldbm_pattern, cell["stats"]):
            ap['strength'] = misc.RunRegex(signaldbm_pattern, cell["stats"])
        elif self.wpa_driver != RALINK_DRIVER:  # This is already set for ralink
            ap['strength'] = -1  

        return ap

    def _connect_to_wpa_ctrl_iface(self):
        """ Connect to the wpa ctrl interface. """
        ctrl_iface = '/var/run/wpa_supplicant'
        socket_loc = os.path.join(ctrl_iface, self.iface)
        if os.path.exists(socket_loc):
            try:
                return wpactrl.WPACtrl(socket_loc)
            except wpactrl.error, e:
                print "Couldn't open ctrl_interface: %s" % e
                return None
        else:
            print "Couldn't find a wpa_supplicant ctrl_interface for iface %s" % self.iface
            return None

    def ValidateAuthentication(self, auth_time):
        """ Validate WPA authentication.

            Validate that the wpa_supplicant authentication
            process was successful.

            NOTE: It's possible this could return False,
            though in reality wpa_supplicant just isn't
            finished yet.

            Keyword arguments:
            auth_time -- The time at which authentication began.

            Returns:
            True if wpa_supplicant authenticated succesfully,
            False otherwise.

        """
        if not WPACTRL_AVAIL:
            # If we don't have python-wpactrl, use the slow version.
            return BaseWirelessInterface.ValidateAuthentication(self, auth_time)
        
        # Right now there's no way to do this for ralink drivers
        if self.wpa_driver == RALINK_DRIVER:
            return True

        wpa = self._connect_to_wpa_ctrl_iface()
        if not wpa:
            print "Failed to open ctrl interface"
            return False

        MAX_TIME = 35
        MAX_DISCONNECTED_TIME = 3
        disconnected_time = 0
        while (time.time() - auth_time) < MAX_TIME:
            try:
                status = wpa.request("STATUS").split("\n")
            except:
                print "wpa_supplicant status query failed."
                return False

            if self.verbose:
                print 'wpa_supplicant ctrl_interface status query is %s' % str(status)

            try:
                [result] = [l for l in status if l.startswith("wpa_state=")]
            except ValueError:
                return False

            if result.endswith("COMPLETED"):
                return True
            elif result.endswith("DISCONNECTED"):
                disconnected_time += 1
                if disconnected_time > MAX_DISCONNECTED_TIME:
                    # Force a rescan to get wpa_supplicant moving again.
                    wpa.request("SCAN")
                    MAX_TIME += 5
            else:
                disconnected_time = 0
            time.sleep(1)

        print 'wpa_supplicant authentication may have failed.'
        return False

    @neediface(False)
    def StopWPA(self):
        """ Terminates wpa_supplicant using its ctrl interface. """
        if not WPACTRL_AVAIL:
            return BaseWirelessInterface.StopWPA(self)
        wpa = self._connect_to_wpa_ctrl_iface()
        if not wpa:
            return
        wpa.request("TERMINATE")

    def _AuthenticateRalinkLegacy(self, network):
        """ Authenticate with the specified wireless network.

        This function handles Ralink legacy cards that cannot use
        wpa_supplicant.

        Keyword arguments:
        network -- dictionary containing network info

        """
        if network.get('key') != None:
            lines = self._GetRalinkInfo()
            for x in lines:
                info = x.split()
                if len(info) < 5:
                    break
                if info[2] == network.get('essid'):
                    if info[5] == 'WEP' or (info[5] == 'OPEN' and \
                                            info[4] == 'WEP'):
                        print 'Setting up WEP'
                        cmd = ''.join(['iwconfig ', self.iface, ' key ',
                                       network.get('key')])
                        if self.verbose: print cmd
                        misc.Run(cmd)
                    else:
                        if info[5] == 'SHARED' and info[4] == 'WEP':
                            print 'Setting up WEP'
                            auth_mode = 'SHARED'
                            key_name = 'Key1'
                        elif info[5] == 'WPA-PSK':
                            print 'Setting up WPA-PSK'
                            auth_mode = 'WPAPSK'
                            key_name = 'WPAPSK'
                        elif info[5] == 'WPA2-PSK':
                            print 'Setting up WPA2-PSK'
                            auth_mode = 'WPA2PSK'
                            key_name = 'WPAPSK'
                        else:
                            print 'Unknown AuthMode, can\'t complete ' + \
                                  'connection process!'
                            return

                        cmd_list = []
                        cmd_list.append('NetworkType=' + info[6])
                        cmd_list.append('AuthMode=' + auth_mode)
                        cmd_list.append('EncrypType=' + info[4])
                        cmd_list.append('SSID=' + info[2])
                        cmd_list.append(key_name + '=' + network.get('key'))
                        if info[5] == 'SHARED' and info[4] == 'WEP':
                            cmd_list.append('DefaultKeyID=1')
                        cmd_list.append('SSID=' + info[2])

                        for cmd in cmd_list:
                            cmd = 'iwpriv ' + self.iface + ' '
                            if self.verbose: print cmd
                            misc.Run(cmd)

    @neediface("")
    def GetBSSID(self, iwconfig=None):
        """ Get the MAC address for the interface. """
        data = (self.iface + '\0' * 32)[:32]
        try:
            result = fcntl.ioctl(self.sock.fileno(), SIOCGIWAP, data)[16:]
        except IOError, e:
            if self.verbose:
                print "SIOCGIWAP failed: " + str(e)
            return ""
        raw_addr = struct.unpack("xxBBBBBB", result[:8])
        return "%02X:%02X:%02X:%02X:%02X:%02X" % raw_addr

    @neediface("")
    def GetCurrentBitrate(self, iwconfig=None):
        """ Get the current bitrate for the interface. """
        data = (self.iface + '\0' * 32)[:32]
        fmt = "ihbb"
        size = struct.calcsize(fmt)
        try:
            result = fcntl.ioctl(self.sock.fileno(), SIOCGIWRATE, data)[16:]
        except IOError, e:
            if self.verbose:
                print "SIOCGIWRATE failed: " + str(e)
            return ""
        f, e, x, x = struct.unpack(fmt, result[:size])
        return "%s %s" % ((f / 1000000), 'Mb/s')

    #def GetOperationalMode(self, iwconfig=None):
    #    """ Get the operational mode for the interface. """
    #    TODO: implement me
    #    return ''

    #def GetAvailableAuthMethods(self, iwlistauth=None):
    #    """ Get the authentication methods for the interface. """
    #    TODO: Implement me
    #    return ''

    @neediface(-1)
    def GetSignalStrength(self, iwconfig=None):
        """ Get the signal strength of the current network.

        Returns:
        The signal strength.

        """
        buff = get_iw_ioctl_result(self.iface, SIOCGIWSTATS)
        strength = ord(buff[2])
        max_strength = self._get_max_strength()
        if strength not in ['', None] and max_strength:
            return 100 * int(strength) // int(max_strength)
        elif strength not in ['', None]:
            return int(strength)
        else:
            return None

    def _get_max_strength(self):
        """ Gets the maximum possible strength from the wireless driver. """
        buff = array.array('c', '\0' * 700)
        addr, length = buff.buffer_info()
        arg = struct.pack('Pi', addr, length)
        iwfreq = (self.iface + '\0' * 16)[:16] + arg
        try:
            result = fcntl.ioctl(self.sock.fileno(), SIOCGIWRANGE, iwfreq)
        except IOError, e:
            if self.verbose:
                print "SIOCGIWRANGE failed: " + str(e)
            return None
        # This defines the iwfreq struct, used to get signal strength.
        fmt = "iiihb6ii4B4Bi32i2i2i2i2i3h8h2b2bhi8i2b3h2i2ihB17x" + 32 * "ihbb"
        size = struct.calcsize(fmt)
        data = buff.tostring()
        data = data[0:size]
        values = struct.unpack(fmt, data)
        return values[12]

    @neediface(-100)
    def GetDBMStrength(self, iwconfig=None):
        """ Get the dBm signal strength of the current network.

        Returns:
        The dBm signal strength.

        """
        buff = get_iw_ioctl_result(self.iface, SIOCGIWSTATS)
        if not buff:
            return None

        return str((ord(buff[3]) - 256))

    @neediface("")
    def GetCurrentNetwork(self, iwconfig=None):
        """ Get the essid of the current network.

        Returns:
        The current network essid.

        """
        buff = get_iw_ioctl_result(self.iface, SIOCGIWESSID)
        if not buff:
            return None

        return buff.strip('\x00')



#!/usr/bin/env python

""" Backend manager for wicd.

Manages and loads the pluggable backends for wicd.

"""

#
#   Copyright (C) 2008-2009 Adam Blackburn
#   Copyright (C) 2008-2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


import sys
import os

import wicd.wpath as wpath

def fail(backend_name, reason):
    print "Failed to load backend %s: %s" % (backend_name, reason)
    return True


class BackendManager(object):
    """ Manages, validates, and loads wicd backends. """
    def __init__(self):
        """ Initialize the backend manager. """
        self.backend_dir = wpath.backends
        self.__loaded_backend = None
        
    def _valid_backend_file(self, be_file):
        """ Make sure the backend file is valid. """
        return (os.path.exists(be_file) and 
                os.path.basename(be_file).startswith("be-") and
                be_file.endswith(".py"))
    
    def get_current_backend(self):
        """ Returns the name of the loaded backend. """
        if self.__loaded_backend:
            return self.__loaded_backend.NAME
        else:
            return None
    
    def get_available_backends(self):
        """ Returns a list of all valid backends in the backend directory. """
        be_list = []
        for f in os.listdir(self.backend_dir):
            if self._valid_backend_file(os.path.join(self.backend_dir, f)):
                be_list.append(f[3:-3])
        return be_list or [""]
    
    def get_update_interval(self):
        """ Returns how often in seconds the wicd monitor should update. """
        if self.__loaded_backend:
            return self.__loaded_backend.UPDATE_INTERVAL
        else:
            return None
        
    def get_backend_description(self, backend_name):
        """ Loads a backend and returns its description. """
        backend = self._load_backend(backend_name)
        if backend and backend.DESCRIPTION:
            return backend.DESCRIPTION
        else:
            return "No backend data available"
    
    def _load_backend(self, backend_name):
        """ Imports a backend and returns the loaded module. """
        print 'trying to load backend %s' % backend_name
        backend_path = os.path.join(self.backend_dir,
                                    'be-' + backend_name + '.py')
        if self._valid_backend_file(backend_path):
            sys.path.insert(0, self.backend_dir)
            backend = __import__('be-' + backend_name)
            return backend
        else:
            fail(backend_name, 'Invalid backend file.')
            return None
        
    def _validate_backend(self, backend, backend_name):
        """ Ensures that a backend module is valid. """
        failed = False
        if not backend.NAME:
            failed = fail(backend_name, 'Missing NAME attribute.')
        if not backend.UPDATE_INTERVAL:
            failed = fail(backend_name, "Missing UPDATE_INTERVAL attribute.")
        if not backend.NeedsExternalCalls:
            failed = fail(backend_name, "Missing NeedsExternalCalls method.")
        if not backend.WiredInterface:
            failed = fail(backend_name, "Missing WiredInterface class.")
        if not backend.WirelessInterface:
            failed = fail(backend_name, "Missing WirelessInterface class.")
        return failed
    
    def load_backend(self, backend_name):
        """ Load and return a backend module. 
        
        Given a backend name be-foo, attempt to load a python module
        in the backends directory called be-foo.py.  The module must
        include a certain set of classes and variables to be considered
        valid.
        
        """
        backend = self._load_backend(backend_name)
        if not backend : return None
        
        failed = self._validate_backend(backend, backend_name)
        if failed:
            return None

        self.__loaded_backend = backend
        print 'successfully loaded backend %s' % backend_name
        return backend



def run_tests():
    import unittest
    test_suite = unittest.TestSuite()

    import testwnettools
    test_suite.addTest(testwnettools.suite())

    import testmisc
    test_suite.addTest(testmisc.suite())

    unittest.TextTestRunner(verbosity=2).run(test_suite)



#!/usr/bin/env python

""" configscript -- Configure the scripts for a particular network.

Script for configuring the scripts for a network passed in as a
command line argument.  This needs to run a separate process because
editing scripts requires root access, and the GUI/Tray are typically
run as the current user.

"""

#
#   Copyright (C) 2007-2009 Adam Blackburn
#   Copyright (C) 2007-2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import sys
import os
import gtk

from wicd import wpath
from wicd.translations import _
from wicd import dbusmanager
from wicd.configmanager import ConfigManager

dbus = dbusmanager.DBusManager()
dbus.connect_to_dbus()

wireless = dbus.get_interface("wireless")
wired = dbus.get_interface("wired")

wireless_conf = wpath.etc + 'wireless-settings.conf'
wired_conf = wpath.etc + 'wired-settings.conf'


def none_to_blank(text):
    """ Converts special string cases to a blank string.
    
    If text is None, 'None', or '' then this method will
    return '', otherwise it will just return str(text).
    
    """
    if text in (None, "None", ""):
        return ""
    else:
        return str(text)

def blank_to_none(text):
    """ Convert an empty or null string to 'None'. """
    if text in ("", None):
        return "None"
    else:
        return str(text)
    
def get_script_info(network, network_type):
    """ Read script info from disk and load it into the configuration dialog """
    info = {}
    if network_type == "wired":
        con = ConfigManager(wired_conf)
        if con.has_section(network):
            info["pre_entry"] = con.get(network, "beforescript", None)
            info["post_entry"] = con.get(network, "afterscript", None)
            info["pre_disconnect_entry"] = con.get(network, "predisconnectscript", None)
            info["post_disconnect_entry"] = con.get(network, "postdisconnectscript", None)
    else:
        bssid = wireless.GetWirelessProperty(int(network), "bssid")
        con = ConfigManager(wireless_conf)
        if con.has_section(bssid):
            info["pre_entry"] = con.get(bssid, "beforescript", None)
            info["post_entry"] = con.get(bssid, "afterscript", None)
            info["pre_disconnect_entry"] = con.get(bssid, "predisconnectscript", None)
            info["post_disconnect_entry"] = con.get(bssid, "postdisconnectscript", None)
    return info

def write_scripts(network, network_type, script_info):
    """ Writes script info to disk and loads it into the daemon. """
    if network_type == "wired":
        con = ConfigManager(wired_conf)
        con.set(network, "beforescript", script_info["pre_entry"])
        con.set(network, "afterscript", script_info["post_entry"])
        con.set(network, "predisconnectscript", script_info["pre_disconnect_entry"])
        con.set(network, "postdisconnectscript", script_info["post_disconnect_entry"])
        con.write(open(wired_conf, "w"))
        wired.ReloadConfig()
        wired.ReadWiredNetworkProfile(network)
        wired.SaveWiredNetworkProfile(network)
    else:
        bssid = wireless.GetWirelessProperty(int(network), "bssid")
        con = ConfigManager(wireless_conf)
        con.set(bssid, "beforescript", script_info["pre_entry"])
        con.set(bssid, "afterscript", script_info["post_entry"])
        con.set(bssid, "predisconnectscript", script_info["pre_disconnect_entry"])
        con.set(bssid, "postdisconnectscript", script_info["post_disconnect_entry"])
        con.write(open(wireless_conf, "w"))
        wireless.ReloadConfig()
        wireless.ReadWirelessNetworkProfile(int(network))
        wireless.SaveWirelessNetworkProfile(int(network))


def main (argv):
    """ Runs the script configuration dialog. """
    if len(argv) < 2:
        print 'Network id to configure is missing, aborting.'
        sys.exit(1)
    
    network = argv[1]
    network_type = argv[2]
    
    script_info = get_script_info(network, network_type)
    
    gladefile = os.path.join(wpath.gtk, "wicd.ui")
    wTree = gtk.Builder()
    wTree.set_translation_domain('wicd')
    wTree.add_from_file(gladefile)
    dialog = wTree.get_object("configure_script_dialog")
    wTree.get_object("pre_label").set_label(_('Pre-connection Script') + ":")
    wTree.get_object("post_label").set_label(_('Post-connection Script') + ":")
    wTree.get_object("pre_disconnect_label").set_label(_('Pre-disconnection Script')
                                                   + ":")
    wTree.get_object("post_disconnect_label").set_label(_('Post-disconnection Script')
                                                   + ":")
    wTree.get_object("window1").hide()
    
    pre_entry = wTree.get_object("pre_entry")
    post_entry = wTree.get_object("post_entry")
    pre_disconnect_entry = wTree.get_object("pre_disconnect_entry")
    post_disconnect_entry = wTree.get_object("post_disconnect_entry")
    
    pre_entry.set_text(none_to_blank(script_info.get("pre_entry")))
    post_entry.set_text(none_to_blank(script_info.get("post_entry")))
    pre_disconnect_entry.set_text(none_to_blank(script_info.get("pre_disconnect_entry")))
    post_disconnect_entry.set_text(none_to_blank(script_info.get("post_disconnect_entry")))

    dialog.show_all()
    
    result = dialog.run()
    if result == 1:
        script_info["pre_entry"] = blank_to_none(pre_entry.get_text())
        script_info["post_entry"] = blank_to_none(post_entry.get_text())
        script_info["pre_disconnect_entry"] = blank_to_none(pre_disconnect_entry.get_text())
        script_info["post_disconnect_entry"] = blank_to_none(post_disconnect_entry.get_text())
        write_scripts(network, network_type, script_info)
    dialog.destroy()
 

if __name__ == '__main__':
    if os.getuid() != 0:
        print "Root privileges are required to configure scripts.  Exiting."
        sys.exit(0)
    main(sys.argv)



#!/usr/bin/python

""" gui -- The main wicd GUI module.

Module containing the code for the main wicd GUI.

"""

#
#   Copyright (C) 2007-2009 Adam Blackburn
#   Copyright (C) 2007-2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import time
import gobject
import pango
import gtk
from itertools import chain
from dbus import DBusException

from wicd import misc
from wicd import wpath
from wicd import dbusmanager
from wicd.misc import noneToString
from wicd.translations import _, language
import prefs
from prefs import PreferencesDialog
import netentry
from netentry import WiredNetworkEntry, WirelessNetworkEntry
from guiutil import error, LabelEntry

if __name__ == '__main__':
    wpath.chdir(__file__)

proxy_obj = daemon = wireless = wired = bus = None
DBUS_AVAIL = False

def setup_dbus(force=True):
    global bus, daemon, wireless, wired, DBUS_AVAIL
    try:
        dbusmanager.connect_to_dbus()
    except DBusException:
        if force:
            print "Can't connect to the daemon, trying to start it automatically..."
            if not misc.PromptToStartDaemon():
                print "Failed to find a graphical sudo program, cannot continue."
                return False
            try:
                dbusmanager.connect_to_dbus()
            except DBusException:
                error(None, _("Could not connect to wicd's D-Bus interface. Check the wicd log for error messages."))
                return False
        else:  
            return False
    prefs.setup_dbus()
    netentry.setup_dbus()
    bus = dbusmanager.get_bus()
    dbus_ifaces = dbusmanager.get_dbus_ifaces()
    daemon = dbus_ifaces['daemon']
    wireless = dbus_ifaces['wireless']
    wired = dbus_ifaces['wired']
    DBUS_AVAIL = True
    
    return True

def handle_no_dbus(from_tray=False):
    global DBUS_AVAIL
    DBUS_AVAIL = False
    if from_tray: return False
    print "Wicd daemon is shutting down!"
    error(None, _('The wicd daemon has shut down. The UI will not function properly until it is restarted.'), block=False)
    return False

        
class WiredProfileChooser:
    """ Class for displaying the wired profile chooser. """
    def __init__(self):
        """ Initializes and runs the wired profile chooser. """
        # Import and init WiredNetworkEntry to steal some of the
        # functions and widgets it uses.
        wired_net_entry = WiredNetworkEntry()

        dialog = gtk.Dialog(title = _('Wired connection detected'),
                            flags = gtk.DIALOG_MODAL,
                            buttons = (gtk.STOCK_CONNECT, 1,
                                       gtk.STOCK_CANCEL, 2))
        dialog.set_has_separator(False)
        dialog.set_size_request(400, 150)
        instruct_label = gtk.Label(_('Select or create a wired profile to connect with') + ':\n')
        stoppopcheckbox = gtk.CheckButton(_('Stop Showing Autoconnect pop-up temporarily'))

        wired_net_entry.is_full_gui = False
        instruct_label.set_alignment(0, 0)
        stoppopcheckbox.set_active(False)

        # Remove widgets that were added to the normal WiredNetworkEntry
        # so that they can be added to the pop-up wizard.
        wired_net_entry.vbox_top.remove(wired_net_entry.hbox_temp)
        wired_net_entry.vbox_top.remove(wired_net_entry.profile_help)

        dialog.vbox.pack_start(instruct_label, fill=False, expand=False)
        dialog.vbox.pack_start(wired_net_entry.profile_help, False, False)
        dialog.vbox.pack_start(wired_net_entry.hbox_temp, False, False)
        dialog.vbox.pack_start(stoppopcheckbox, False, False)
        dialog.show_all()

        wired_profiles = wired_net_entry.combo_profile_names
        wired_net_entry.profile_help.hide()
        if wired_net_entry.profile_list != None:
            wired_profiles.set_active(0)
            print "wired profiles found"
        else:
            print "no wired profiles found"
            wired_net_entry.profile_help.show()

        response = dialog.run()
        if response == 1:
            print 'reading profile ', wired_profiles.get_active_text()
            wired.ReadWiredNetworkProfile(wired_profiles.get_active_text())
            wired.ConnectWired()
        else:
            if stoppopcheckbox.get_active():
                daemon.SetForcedDisconnect(True)
        dialog.destroy()


def get_wireless_prop(net_id, prop):
    return wireless.GetWirelessProperty(net_id, prop)

class appGui(object):
    """ The main wicd GUI class. """
    def __init__(self, standalone=False, tray=None):
        """ Initializes everything needed for the GUI. """
        setup_dbus()

        if not daemon:
            errmsg = "Error connecting to wicd service via D-Bus." + \
                     "Please ensure the wicd service is running."
            d = gtk.MessageDialog(parent=None,
                                  flags=gtk.DIALOG_MODAL,
                                  type=gtk.MESSAGE_ERROR,
                                  buttons=gtk.BUTTONS_OK,
                                  message_format=errmsg)
            d.run()
            sys.exit(1)

        self.tray = tray

        gladefile = os.path.join(wpath.gtk, "wicd.ui")
        self.wTree = gtk.Builder()
        self.wTree.set_translation_domain('wicd')
        self.wTree.add_from_file(gladefile)
        self.window = self.wTree.get_object("window1")
        width = int(gtk.gdk.screen_width() / 2)
        if width > 530:
            width = 530
        self.window.resize(width, int(gtk.gdk.screen_height() / 1.7))

        dic = { "refresh_clicked" : self.refresh_clicked, 
                "quit_clicked" : self.exit,
                "rfkill_clicked" : self.switch_rfkill,
                "disconnect_clicked" : self.disconnect_all,
                "main_exit" : self.exit, 
                "cancel_clicked" : self.cancel_connect,
                "hidden_clicked" : self.connect_hidden,
                "preferences_clicked" : self.settings_dialog,
                "about_clicked" : self.about_dialog,
                "create_adhoc_clicked" : self.create_adhoc_network,
                }
        self.wTree.connect_signals(dic)

        # Set some strings in the GUI - they may be translated
        label_instruct = self.wTree.get_object("label_instructions")
        label_instruct.set_label(_('Choose from the networks below:'))

        probar = self.wTree.get_object("progressbar")
        probar.set_text(_('Connecting'))

        self.rfkill_button = self.wTree.get_object("rfkill_button")
        self.all_network_list = self.wTree.get_object("network_list_vbox")
        self.all_network_list.show_all()
        self.wired_network_box = gtk.VBox(False, 0)
        self.wired_network_box.show_all()
        self.network_list = gtk.VBox(False, 0)
        self.all_network_list.pack_start(self.wired_network_box, False, False)
        self.all_network_list.pack_start(self.network_list, True, True)
        self.network_list.show_all()
        self.status_area = self.wTree.get_object("connecting_hbox")
        self.status_bar = self.wTree.get_object("statusbar")
        menu = self.wTree.get_object("menu1")

        self.status_area.hide_all()

        if os.path.exists(os.path.join(wpath.images, "wicd.png")):
            self.window.set_icon_from_file(os.path.join(wpath.images, "wicd.png"))
        self.statusID = None
        self.first_dialog_load = True
        self.is_visible = True
        self.pulse_active = False
        self.pref = None
        self.standalone = standalone
        self.wpadrivercombo = None
        self.connecting = False
        self.refreshing = False
        self.prev_state = None
        self.update_cb = None
        self._wired_showing = False
        self.network_list.set_sensitive(False)
        label = gtk.Label("%s..." % _('Scanning'))
        self.network_list.pack_start(label)
        label.show()
        self.wait_for_events(0.2)
        self.window.connect('delete_event', self.exit)
        self.window.connect('key-release-event', self.key_event)
        daemon.SetGUIOpen(True)
        bus.add_signal_receiver(self.dbus_scan_finished, 'SendEndScanSignal',
                        'org.wicd.daemon.wireless')
        bus.add_signal_receiver(self.dbus_scan_started, 'SendStartScanSignal',
                        'org.wicd.daemon.wireless')
        bus.add_signal_receiver(self.update_connect_buttons, 'StatusChanged',
                        'org.wicd.daemon')
        bus.add_signal_receiver(self.handle_connection_results,
                                'ConnectResultsSent', 'org.wicd.daemon')
        bus.add_signal_receiver(lambda: setup_dbus(force=False), 
                                "DaemonStarting", "org.wicd.daemon")
        bus.add_signal_receiver(self._do_statusbar_update, 'StatusChanged',
                                'org.wicd.daemon')
        if standalone:
            bus.add_signal_receiver(handle_no_dbus, "DaemonClosing", 
                                    "org.wicd.daemon")
            
        self._do_statusbar_update(*daemon.GetConnectionStatus())
        self.wait_for_events(0.1)
        self.update_cb = misc.timeout_add(2, self.update_statusbar)
        self.refresh_clicked()
        
    def handle_connection_results(self, results):
        if results not in ['success', 'aborted'] and self.is_visible:
            error(self.window, language[results], block=False)

    def create_adhoc_network(self, widget=None):
        """ Shows a dialog that creates a new adhoc network. """
        print "Starting the Ad-Hoc Network Creation Process..."
        dialog = gtk.Dialog(title = _('Create an Ad-Hoc Network'),
                            flags = gtk.DIALOG_MODAL,
                            buttons=(gtk.STOCK_CANCEL, 2, gtk.STOCK_OK, 1))
        dialog.set_has_separator(False)
        dialog.set_size_request(400, -1)
        self.chkbox_use_encryption = gtk.CheckButton(_('Use Encryption (WEP only)'))
        self.chkbox_use_encryption.set_active(False)
        ip_entry = LabelEntry(_('IP') + ':')
        essid_entry = LabelEntry(_('ESSID') + ':')
        channel_entry = LabelEntry(_('Channel') + ':')
        self.key_entry = LabelEntry(_('Key') + ':')
        self.key_entry.set_auto_hidden(True)
        self.key_entry.set_sensitive(False)

        chkbox_use_ics = gtk.CheckButton( _('Activate Internet Connection Sharing'))

        self.chkbox_use_encryption.connect("toggled",
                                           self.toggle_encrypt_check)
        channel_entry.entry.set_text('3')
        essid_entry.entry.set_text('My_Adhoc_Network')
        ip_entry.entry.set_text('169.254.12.10')  # Just a random IP

        vbox_ah = gtk.VBox(False, 0)
        self.wired_network_box = gtk.VBox(False, 0)
        vbox_ah.pack_start(self.chkbox_use_encryption, False, False)
        vbox_ah.pack_start(self.key_entry, False, False)
        vbox_ah.show()
        dialog.vbox.pack_start(essid_entry)
        dialog.vbox.pack_start(ip_entry)
        dialog.vbox.pack_start(channel_entry)
        dialog.vbox.pack_start(chkbox_use_ics)
        dialog.vbox.pack_start(vbox_ah)
        dialog.vbox.set_spacing(5)
        dialog.show_all()
        response = dialog.run()
        if response == 1:
            wireless.CreateAdHocNetwork(essid_entry.entry.get_text(),
                                        channel_entry.entry.get_text(),
                                        ip_entry.entry.get_text().strip(),
                                        "WEP",
                                        self.key_entry.entry.get_text(),
                                        self.chkbox_use_encryption.get_active(),
                                        False) #chkbox_use_ics.get_active())
        dialog.destroy()

    def toggle_encrypt_check(self, widget=None):
        """ Toggles the encryption key entry box for the ad-hoc dialog. """
        self.key_entry.set_sensitive(self.chkbox_use_encryption.get_active())

    def switch_rfkill(self, widget=None):
        """ Switches wifi card on/off. """
        wireless.SwitchRfKill()
        if wireless.GetRfKillEnabled():
            self.rfkill_button.set_stock_id(gtk.STOCK_MEDIA_PLAY)
            self.rfkill_button.set_label(_('Switch On Wi-Fi'))
        else:
            self.rfkill_button.set_stock_id(gtk.STOCK_MEDIA_STOP)
            self.rfkill_button.set_label(_('Switch Off Wi-Fi'))

    def disconnect_all(self, widget=None):
        """ Disconnects from any active network. """
        def handler(*args):
            gobject.idle_add(self.all_network_list.set_sensitive, True)
         
        self.all_network_list.set_sensitive(False)
        daemon.Disconnect(reply_handler=handler, error_handler=handler)

    def about_dialog(self, widget, event=None):
        """ Displays an about dialog. """
        dialog = gtk.AboutDialog()
        dialog.set_name("Wicd")
        dialog.set_version(daemon.Hello())
        dialog.set_authors([ "Adam Blackburn", "Dan O'Reilly", "Andrew Psaltis", "David Paleino"])
        dialog.set_website("http://wicd.sourceforge.net")
        dialog.run()
        dialog.destroy()
        
    def key_event (self, widget, event=None):
        """ Handle key-release-events. """
        if event.state & gtk.gdk.CONTROL_MASK and \
           gtk.gdk.keyval_name(event.keyval) in ["w", "q"]:
            self.exit()
    
    def settings_dialog(self, widget, event=None):
        """ Displays a general settings dialog. """
        if not self.pref:
            self.pref = PreferencesDialog(self, self.wTree)
        else:
            self.pref.load_preferences_diag()
        if self.pref.run() == 1:
            self.pref.save_results()
        self.pref.hide()

    def connect_hidden(self, widget):
        """ Prompts the user for a hidden network, then scans for it. """
        dialog = gtk.Dialog(title=('Hidden Network'),
                            flags=gtk.DIALOG_MODAL,
                            buttons=(gtk.STOCK_CONNECT, 1, gtk.STOCK_CANCEL, 2))
        dialog.set_has_separator(False)
        lbl = gtk.Label(_('Hidden Network ESSID'))
        textbox = gtk.Entry()
        dialog.vbox.pack_start(lbl)
        dialog.vbox.pack_start(textbox)
        dialog.show_all()
        button = dialog.run()
        if button == 1:
            answer = textbox.get_text()
            dialog.destroy()
            self.refresh_networks(None, True, answer)
        else:
            dialog.destroy()

    def cancel_connect(self, widget):
        """ Alerts the daemon to cancel the connection process. """
        #should cancel a connection if there
        #is one in progress
        cancel_button = self.wTree.get_object("cancel_button")
        cancel_button.set_sensitive(False)
        daemon.CancelConnect()
        # Prevents automatic reconnecting if that option is enabled
        daemon.SetForcedDisconnect(True)

    def pulse_progress_bar(self):
        """ Pulses the progress bar while connecting to a network. """
        if not self.pulse_active:
            return False
        if not self.is_visible:
            return True
        try:
            gobject.idle_add(self.wTree.get_object("progressbar").pulse)
        except:
            pass
        return True

    def update_statusbar(self):
        """ Triggers a status update in wicd-monitor. """
        if not self.is_visible:
            return True
        
        daemon.UpdateState()
        if self.connecting:
            # If we're connecting, don't wait for the monitor to send
            # us a signal, since it won't until the connection is made.
            self._do_statusbar_update(*daemon.GetConnectionStatus())
        return True
    
    def _do_statusbar_update(self, state, info):
        if not self.is_visible:
            return True
        
        if state == misc.WIRED:
            return self.set_wired_state(info)
        elif state == misc.WIRELESS:
            return self.set_wireless_state(info)
        elif state == misc.CONNECTING:
            return self.set_connecting_state(info)
        elif state in (misc.SUSPENDED, misc.NOT_CONNECTED):
            return self.set_not_connected_state(info)
        return True
        
    def set_wired_state(self, info):
        if self.connecting:
            # Adjust our state from connecting->connected.
            self._set_not_connecting_state()
        self.set_status(_('Connected to wired network (IP: $A)').replace('$A', info[0]))
        return True
    
    def set_wireless_state(self, info):
        if self.connecting:
            # Adjust our state from connecting->connected.
            self._set_not_connecting_state()
        self.set_status(_('Connected to $A at $B (IP: $C)').replace
                        ('$A', info[1]).replace
                        ('$B', daemon.FormatSignalForPrinting(info[2])).replace
                        ('$C', info[0]))
        return True
        
    def set_not_connected_state(self, info):
        if self.connecting:
            # Adjust our state from connecting->not-connected.
            self._set_not_connecting_state()
        self.set_status(_('Not connected'))
        return True
        
    def _set_not_connecting_state(self):
        if self.connecting:
            if self.update_cb:
                gobject.source_remove(self.update_cb)
            self.update_cb = misc.timeout_add(2, self.update_statusbar)
            self.connecting = False
        if self.pulse_active:
            self.pulse_active = False
            gobject.idle_add(self.all_network_list.set_sensitive, True)
            gobject.idle_add(self.status_area.hide_all)
        if self.statusID:
            gobject.idle_add(self.status_bar.remove_message, 1, self.statusID)
    
    def set_connecting_state(self, info):
        if not self.connecting:
            if self.update_cb:
                gobject.source_remove(self.update_cb)
            self.update_cb = misc.timeout_add(500, self.update_statusbar, 
                                              milli=True)
            self.connecting = True
        if not self.pulse_active:
            self.pulse_active = True
            misc.timeout_add(100, self.pulse_progress_bar, milli=True)
            gobject.idle_add(self.all_network_list.set_sensitive, False)
            gobject.idle_add(self.status_area.show_all)
        if self.statusID:
            gobject.idle_add(self.status_bar.remove_message, 1, self.statusID)
        if info[0] == "wireless":
            stat = wireless.CheckWirelessConnectingMessage()
            gobject.idle_add(self.set_status, "%s: %s" % (info[1], stat))
        elif info[0] == "wired":
            gobject.idle_add(self.set_status, _('Wired Network') + ': ' \
                + wired.CheckWiredConnectingMessage())
        return True
        
    def update_connect_buttons(self, state=None, x=None, force_check=False):
        """ Updates the connect/disconnect buttons for each network entry.

        If force_check is given, update the buttons even if the
        current network state is the same as the previous.
        
        """
        if not DBUS_AVAIL: return
        if not state:
            state, x = daemon.GetConnectionStatus()
        
        if self.prev_state != state or force_check:
            apbssid = wireless.GetApBssid()
            for entry in chain(self.network_list, self.wired_network_box):
                if hasattr(entry, "update_connect_button"):
                    entry.update_connect_button(state, apbssid)
        self.prev_state = state
    
    def set_status(self, msg):
        """ Sets the status bar message for the GUI. """
        self.statusID = self.status_bar.push(1, msg)
        
    def dbus_scan_finished(self):
        """ Calls for a non-fresh update of the gui window.
        
        This method is called after a wireless scan is completed.
        
        """
        if not DBUS_AVAIL: return
        gobject.idle_add(self.refresh_networks, None, False, None)
            
    def dbus_scan_started(self):
        """ Called when a wireless scan starts. """
        if not DBUS_AVAIL: return
        self.network_list.set_sensitive(False)

    def _remove_items_from_vbox(self, vbox):
        for z in vbox:
            vbox.remove(z)
            z.destroy()
            del z

    
    def refresh_clicked(self, widget=None):
        """ Kick off an asynchronous wireless scan. """
        if not DBUS_AVAIL or self.connecting: return
        self.refreshing = True

        # Remove stuff already in there.
        self._remove_items_from_vbox(self.wired_network_box)
        self._remove_items_from_vbox(self.network_list)
        label = gtk.Label("%s..." % _('Scanning'))
        self.network_list.pack_start(label)
        self.network_list.show_all()
        if wired.CheckPluggedIn() or daemon.GetAlwaysShowWiredInterface():
            printLine = True  # In this case we print a separator.
            wirednet = WiredNetworkEntry()
            self.wired_network_box.pack_start(wirednet, False, False)
            wirednet.connect_button.connect("clicked", self.connect,
                                           "wired", 0, wirednet)
            wirednet.disconnect_button.connect("clicked", self.disconnect,
                                               "wired", 0, wirednet)
            wirednet.advanced_button.connect("clicked",
                                             self.edit_advanced, "wired", 0, 
                                             wirednet)
            state, x = daemon.GetConnectionStatus()
            wirednet.update_connect_button(state)

            self._wired_showing = True
        else:
            self._wired_showing = False

        wireless.Scan(False)

    def refresh_networks(self, widget=None, fresh=True, hidden=None):
        """ Refreshes the network list.
        
        If fresh=True, scans for wireless networks and displays the results.
        If a ethernet connection is available, or the user has chosen to,
        displays a Wired Network entry as well.
        If hidden isn't None, will scan for networks after running
        iwconfig <wireless interface> essid <hidden>.
        
        """
        if fresh:
            if hidden:
                wireless.SetHiddenNetworkESSID(noneToString(hidden))
            self.refresh_clicked()
            return
        print "refreshing..."
        self.network_list.set_sensitive(False)
        self._remove_items_from_vbox(self.network_list)
        self.wait_for_events()
        printLine = False  # We don't print a separator by default.
        if self._wired_showing:
            printLine = True
        num_networks = wireless.GetNumberOfNetworks()
        instruct_label = self.wTree.get_object("label_instructions")
        if num_networks > 0:
            skip_never_connect = not daemon.GetShowNeverConnect()
            instruct_label.show()
            for x in xrange(0, num_networks):
                if skip_never_connect and misc.to_bool(get_wireless_prop(x,'never')): continue
                if printLine:
                    sep = gtk.HSeparator()
                    self.network_list.pack_start(sep, padding=10, fill=False,
                                                 expand=False)
                    sep.show()
                else:
                    printLine = True
                tempnet = WirelessNetworkEntry(x)
                self.network_list.pack_start(tempnet, False, False)
                tempnet.connect_button.connect("clicked",
                                               self.connect, "wireless", x,
                                               tempnet)
                tempnet.disconnect_button.connect("clicked",
                                                  self.disconnect, "wireless",
                                                  x, tempnet)
                tempnet.advanced_button.connect("clicked",
                                                self.edit_advanced, "wireless",
                                                x, tempnet)
        else:
            instruct_label.hide()
            if wireless.GetKillSwitchEnabled():
                label = gtk.Label(_('Wireless Kill Switch Enabled') + ".")
            else:
                label = gtk.Label(_('No wireless networks found.'))
            self.network_list.pack_start(label)
            label.show()
        self.update_connect_buttons(force_check=True)
        self.network_list.set_sensitive(True)
        self.refreshing = False

    def save_settings(self, nettype, networkid, networkentry):
        """ Verifies and saves the settings for the network entry. """
        entry = networkentry.advanced_dialog
        opt_entlist = []
        req_entlist = []
        
        # First make sure all the Addresses entered are valid.
        if entry.chkbox_static_ip.get_active():
            req_entlist = [entry.txt_ip, entry.txt_netmask]
            opt_entlist = [entry.txt_gateway]
                
        if entry.chkbox_static_dns.get_active() and \
           not entry.chkbox_global_dns.get_active():
            for ent in [entry.txt_dns_1, entry.txt_dns_2, entry.txt_dns_3]:
                opt_entlist.append(ent)
        
        # Required entries.
        for lblent in req_entlist:
            lblent.set_text(lblent.get_text().strip())
            if not misc.IsValidIP(lblent.get_text()):
                error(self.window, _('Invalid address in $A entry.').
                                    replace('$A', lblent.label.get_label()))
                return False
        
        # Optional entries, only check for validity if they're entered.
        for lblent in opt_entlist:
            lblent.set_text(lblent.get_text().strip())
            if lblent.get_text() and not misc.IsValidIP(lblent.get_text()):
                error(self.window, _('Invalid address in $A entry.').
                                    replace('$A', lblent.label.get_label()))
                return False

        # Now save the settings.
        if nettype == "wireless":
            if not networkentry.save_wireless_settings(networkid):
                return False

        elif nettype == "wired":
            if not networkentry.save_wired_settings():
                return False
            
        return True

    def edit_advanced(self, widget, ttype, networkid, networkentry):
        """ Display the advanced settings dialog.
        
        Displays the advanced settings dialog and saves any changes made.
        If errors occur in the settings, an error message will be displayed
        and the user won't be able to save the changes until the errors
        are fixed.
        
        """
        dialog = networkentry.advanced_dialog
        dialog.set_values()
        dialog.show_all()
        while True:
            if self.run_settings_dialog(dialog, ttype, networkid, networkentry):
                break
        dialog.hide()
        
    def run_settings_dialog(self, dialog, nettype, networkid, networkentry):
        """ Runs the settings dialog.
        
        Runs the settings dialog and returns True if settings are saved
        successfully, and false otherwise.
        
        """
        result = dialog.run()
        if result == gtk.RESPONSE_ACCEPT:
            if self.save_settings(nettype, networkid, networkentry):
                return True
            else:
                return False
        return True
    
    def check_encryption_valid(self, networkid, entry):
        """ Make sure that encryption settings are properly filled in. """
        # Make sure no entries are left blank
        if entry.chkbox_encryption.get_active():
            encryption_info = entry.encryption_info
            for entry_info in encryption_info.itervalues():
                if entry_info[0].entry.get_text() == "" and \
                   entry_info[1] == 'required':
                    error(self.window, "%s (%s)" % (_('Required encryption information is missing.'),
                                             entry_info[0].label.get_label())
                          )
                    return False
        # Make sure the checkbox is checked when it should be
        elif not entry.chkbox_encryption.get_active() and \
             wireless.GetWirelessProperty(networkid, "encryption"):
            error(self.window, _('This network requires encryption to be enabled.'))
            return False
        return True

    def _wait_for_connect_thread_start(self):
        self.wTree.get_object("progressbar").pulse()
        if not self._connect_thread_started:
            return True
        else:
            misc.timeout_add(2, self.update_statusbar)
            self.update_statusbar()
            return False
        
    def connect(self, widget, nettype, networkid, networkentry):
        """ Initiates the connection process in the daemon. """
        def handler(*args):
            self._connect_thread_started = True

        def setup_interface_for_connection():
            cancel_button = self.wTree.get_object("cancel_button")
            cancel_button.set_sensitive(True)
            self.all_network_list.set_sensitive(False)
            if self.statusID:
                gobject.idle_add(self.status_bar.remove_message, 1, self.statusID)
            gobject.idle_add(self.set_status, _('Disconnecting active connections...'))
            gobject.idle_add(self.status_area.show_all)
            self.wait_for_events()
            self._connect_thread_started = False

        if nettype == "wireless":
            if not self.check_encryption_valid(networkid,
                                               networkentry.advanced_dialog):
                self.edit_advanced(None, nettype, networkid, networkentry)
                return False
            setup_interface_for_connection()
            wireless.ConnectWireless(networkid, reply_handler=handler,
                                     error_handler=handler)
        elif nettype == "wired":
            setup_interface_for_connection()
            wired.ConnectWired(reply_handler=handler, error_handler=handler)
        
        gobject.source_remove(self.update_cb)
        misc.timeout_add(100, self._wait_for_connect_thread_start, milli=True)
        
    def disconnect(self, widget, nettype, networkid, networkentry):
        """ Disconnects from the given network.
        
        Keyword arguments:
        widget -- The disconnect button that was pressed.
        event -- unused
        nettype -- "wired" or "wireless", depending on the network entry type.
        networkid -- unused
        networkentry -- The NetworkEntry containing the disconnect button.
        
        """
        def handler(*args):
            gobject.idle_add(self.all_network_list.set_sensitive, True)
            gobject.idle_add(self.network_list.set_sensitive, True)
            
        widget.hide()
        networkentry.connect_button.show()
        daemon.SetForcedDisconnect(True)
        self.network_list.set_sensitive(False)
        if nettype == "wired":
            wired.DisconnectWired(reply_handler=handler, error_handler=handler)
        else:
            wireless.DisconnectWireless(reply_handler=handler, 
                                        error_handler=handler)
        
    def wait_for_events(self, amt=0):
        """ Wait for any pending gtk events to finish before moving on. 

        Keyword arguments:
        amt -- a number specifying the number of ms to wait before checking
               for pending events.
        
        """
        time.sleep(amt)
        while gtk.events_pending():
            gtk.main_iteration()

    def exit(self, widget=None, event=None):
        """ Hide the wicd GUI.

        This method hides the wicd GUI and writes the current window size
        to disc for later use.  This method normally does NOT actually 
        destroy the GUI, it just hides it.

        """
        self.window.hide()
        gobject.source_remove(self.update_cb)
        bus.remove_signal_receiver(self._do_statusbar_update, 'StatusChanged',
                                   'org.wicd.daemon')
        [width, height] = self.window.get_size()
        try:
            daemon.SetGUIOpen(False)
        except DBusException:
            pass

        if self.standalone:
            sys.exit(0)

        self.is_visible = False
        return True

    def show_win(self):
        """ Brings the GUI out of the hidden state. 
        
        Method to show the wicd GUI, alert the daemon that it is open,
        and refresh the network list.
        
        """
        self.window.present()
        self.window.deiconify()
        self.wait_for_events()
        self.is_visible = True
        daemon.SetGUIOpen(True)
        self.wait_for_events(0.1)
        gobject.idle_add(self.refresh_clicked)
        self._do_statusbar_update(*daemon.GetConnectionStatus())
        bus.add_signal_receiver(self._do_statusbar_update, 'StatusChanged',
                                'org.wicd.daemon')
        self.update_cb = misc.timeout_add(2, self.update_statusbar)


if __name__ == '__main__':
    setup_dbus()
    app = appGui(standalone=True)
    mainloop = gobject.MainLoop()
    mainloop.run()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

""" Network interface control tools for wicd.

This module implements functions to control and obtain information from
network interfaces.

class Interface() -- Control a network interface.
class WiredInterface() -- Control a wired network interface.
class WirelessInterface() -- Control a wireless network interface.

"""

#
#   Copyright (C) 2008-2009 Adam Blackburn
#   Copyright (C) 2008-2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from wicd.wnettools import GetDefaultGateway, GetWiredInterfaces, \
GetWirelessInterfaces, IsValidWpaSuppDriver, BaseWirelessInterface, \
BaseWiredInterface, BaseInterface, GetWpaSupplicantDrivers

NAME = "external"
UPDATE_INTERVAL = 5
DESCRIPTION = """External app (original) backend

This backend uses external program calls like ifconfig and
iwconfig to query network information.  This makes it a bit
slower and more CPU intensive than the ioctl backend, but
it doesn't require any third party libraries and may be
more stable for some set ups.
"""


def NeedsExternalCalls(*args, **kargs):
    """ Return True, since this backend uses iwconfig/ifconfig. """
    return True


class Interface(BaseInterface):
    """ Control a network interface. """
    def __init__(self, iface, verbose=False):
        """ Initialize the object.

        Keyword arguments:
        iface -- the name of the interface
        verbose -- whether to print every command run

        """
        BaseInterface.__init__(self, iface, verbose)
        self.Check()
    

class WiredInterface(Interface, BaseWiredInterface):
    """ Control a wired network interface. """
    def __init__(self, iface, verbose=False):
        """ Initialise the wired network interface class.

        Keyword arguments:
        iface -- name of the interface
        verbose -- print all commands

        """
        BaseWiredInterface.__init__(self, iface, verbose)
        Interface.__init__(self, iface, verbose)


class WirelessInterface(Interface, BaseWirelessInterface):
    """ Control a wireless network interface. """
    def __init__(self, iface, verbose=False, wpa_driver='wext'):
        """ Initialise the wireless network interface class.

        Keyword arguments:
        iface -- name of the interface
        verbose -- print all commands

        """
        BaseWirelessInterface.__init__(self, iface, verbose, wpa_driver)
        Interface.__init__(self, iface, verbose)



#!/usr/bin/python

#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import optparse
import dbus
import dbus.service
import sys
from wicd import misc

misc.RenameProcess('wicd-cli')

if getattr(dbus, 'version', (0, 0, 0)) < (0, 80, 0):
    import dbus.glib
else:
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)

bus = dbus.SystemBus()
try:
    daemon = dbus.Interface(bus.get_object('org.wicd.daemon', '/org/wicd/daemon'),
            'org.wicd.daemon')
    wireless = dbus.Interface(bus.get_object('org.wicd.daemon', '/org/wicd/daemon/wireless'),
            'org.wicd.daemon.wireless')
    wired = dbus.Interface(bus.get_object('org.wicd.daemon', '/org/wicd/daemon/wired'),
            'org.wicd.daemon.wired')
    config = dbus.Interface(bus.get_object('org.wicd.daemon', '/org/wicd/daemon/config'),
            'org.wicd.daemon.config')
except dbus.DBusException:
    print 'Error: Could not connect to the daemon. Please make sure it is running.'
    sys.exit(3)

if not daemon:
    print 'Error connecting to wicd via D-Bus.  Please make sure the wicd service is running.'
    sys.exit(3)

parser = optparse.OptionParser()

parser.add_option('--network', '-n', type='int', default=-1)
parser.add_option('--network-property', '-p')
parser.add_option('--set-to', '-s')
parser.add_option('--name', '-m')

parser.add_option('--scan', '-S', default=False, action='store_true')
parser.add_option('--save', '-w', default=False, action='store_true')
parser.add_option('--list-networks', '-l', default=False, action='store_true')
parser.add_option('--network-details', '-d', default=False, action='store_true')
parser.add_option('--disconnect', '-x', default=False, action='store_true')
parser.add_option('--connect', '-c', default=False, action='store_true')
parser.add_option('--list-encryption-types', '-e', default=False, action='store_true')
# short options for these two aren't great.
parser.add_option('--wireless', '-y', default=False, action='store_true')
parser.add_option('--wired', '-z', default=False, action='store_true')
parser.add_option('--load-profile', '-o', default=False, action='store_true')

options, arguments = parser.parse_args()

op_performed = False

if not (options.wireless or options.wired):
    print "Please use --wireless or --wired to specify " + \
    "the type of connection to operate on."

# functions
def is_valid_wireless_network_id(network_id):
    if not (network_id >= 0 \
            and network_id < wireless.GetNumberOfNetworks()):
        print 'Invalid wireless network identifier.'
        sys.exit(1)

def is_valid_wired_network_id(network_id):
    num = len(wired.GetWiredProfileList())
    if not (network_id < num and \
            network_id >= 0):
        print 'Invalid wired network identifier.'
        sys.exit(4)

def is_valid_wired_network_profile(profile_name):
    if not profile_name in wired.GetWiredProfileList():
        print 'Profile of that name does not exist.'
        sys.exit(5)

if options.scan and options.wireless:
    # synchronized scan
    wireless.Scan(True)
    op_performed = True

if options.load_profile and options.wired:
    is_valid_wired_network_profile(options.name)
    wired.ReadWiredNetworkProfile(options.name)
    op_performed = True

if options.list_networks:
    if options.wireless:
        print '#\tBSSID\t\t\tChannel\tESSID'
        for network_id in range(0, wireless.GetNumberOfNetworks()):
            print '%s\t%s\t%s\t%s' % (network_id,
                wireless.GetWirelessProperty(network_id, 'bssid'),
                wireless.GetWirelessProperty(network_id, 'channel'),
                wireless.GetWirelessProperty(network_id, 'essid'))
    elif options.wired:
        print '#\tProfile name'
        id = 0
        for profile in wired.GetWiredProfileList():
            print '%s\t%s' % (id, profile)
            id += 1
    op_performed = True

if options.network_details:
    if options.wireless:
        if options.network >= 0:
            is_valid_wireless_network_id(options.network)
            network_id = options.network
        else:
            network_id = wireless.GetCurrentNetworkID(0)
            is_valid_wireless_network_id(network_id)
            # we're connected to a network, print IP
            print "IP: %s" % wireless.GetWirelessIP(0)

        print "Essid: %s" % wireless.GetWirelessProperty(network_id, "essid")
        print "Bssid: %s" % wireless.GetWirelessProperty(network_id, "bssid")
        if wireless.GetWirelessProperty(network_id, "encryption"):
            print "Encryption: On"
            print "Encryption Method: %s" % \
                    wireless.GetWirelessProperty(network_id, "encryption_method")
        else:
            print "Encryption: Off"
        print "Quality: %s" % wireless.GetWirelessProperty(network_id, "quality")
        print "Mode: %s" % wireless.GetWirelessProperty(network_id, "mode")
        print "Channel: %s" % wireless.GetWirelessProperty(network_id, "channel")
        print "Bit Rates: %s" % wireless.GetWirelessProperty(network_id, "bitrates")
    op_performed = True

# network properties

if options.network_property:
    options.network_property = options.network_property.lower()
    if options.wireless:
        if options.network >= 0:
            is_valid_wireless_network_id(options.network)
            network_id = options.network
        else:
            network_id = wireless.GetCurrentNetworkID(0)
            is_valid_wireless_network_id(network_id)
        if not options.set_to:
            print wireless.GetWirelessProperty(network_id, options.network_property)
        else:
            wireless.SetWirelessProperty(network_id, \
                    options.network_property, options.set_to)
    elif options.wired:
        if not options.set_to:
            print wired.GetWiredProperty(options.network_property)
        else:
            wired.SetWiredProperty(options.network_property, options.set_to)
    op_performed = True

if options.disconnect:
    daemon.Disconnect()
    if options.wireless:
        if wireless.GetCurrentNetworkID(0) > -1:
            print "Disconnecting from %s on %s" % (wireless.GetCurrentNetwork(0),
                    wireless.DetectWirelessInterface())
    elif options.wired:
        if wired.CheckPluggedIn():
            print "Disconnecting from wired connection on %s" % wired.DetectWiredInterface()
    op_performed = True

if options.connect:
    check = None
    if options.wireless and options.network > -1:
        is_valid_wireless_network_id(options.network)
        name = wireless.GetWirelessProperty(options.network, 'essid')
        encryption = wireless.GetWirelessProperty(options.network, 'enctype')
        print "Connecting to %s with %s on %s" % (name, encryption,
                wireless.DetectWirelessInterface())
        wireless.ConnectWireless(options.network)

        check = lambda: wireless.CheckIfWirelessConnecting()
        status = lambda: wireless.CheckWirelessConnectingStatus()
        message = lambda: wireless.CheckWirelessConnectingMessage()
    elif options.wired:
        print "Connecting to wired connection on %s" % wired.DetectWiredInterface()
        wired.ConnectWired()

        check = lambda: wired.CheckIfWiredConnecting()
        status = lambda: wired.CheckWiredConnectingStatus()
        message = lambda: wired.CheckWiredConnectingMessage()
    else:
        check = lambda: False
        status = lambda: False
        message = lambda: False

    # update user on what the daemon is doing
    last = None
    if check:
        while check():
            next = status()
            if next != last:
                # avoid a race condition where status is updated to "done" after the
                # loop check
                if next == "done":
                    break
                print message()
                last = next
        print "done!"
        op_performed = True

# pretty print optional and required properties
def str_properties(prop):
    if len(prop) == 0:
        return "None"
    else:
        return ', '.join("%s (%s)" % (x[0], x[1].replace("_", " ")) for x in type['required'])

if options.wireless and options.list_encryption_types:
    et = misc.LoadEncryptionMethods()
    # print 'Installed encryption templates:'
    print '%s\t%-20s\t%s' % ('#', 'Name', 'Description')
    id = 0
    for type in et:
        print '%s\t%-20s\t%s' % (id, type['type'], type['name'])
        print '  Req: %s' % str_properties(type['required'])
        print '---'
        # don't print optionals (yet)
        #print '  Opt: %s' % str_properties(type['optional'])
        id += 1
    op_performed = True

if options.save and options.network > -1:
    if options.wireless:
        is_valid_wireless_network_id(options.network)
        config.SaveWirelessNetworkProfile(options.network)
    elif options.wired:
        config.SaveWiredNetworkProfile(options.name)
    op_performed = True

if not op_performed:
    print "No operations performed."




#!/usr/bin/env python

""" The wicd DBus Manager.

A module for managing wicd's dbus interfaces.

"""

#
#   Copyright (C) 2008-2009 Adam Blackburn
#   Copyright (C) 2008-2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus
if getattr(dbus, "version", (0, 0, 0)) < (0, 80, 0):
    import dbus.glib
else:
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)
    
DBUS_MANAGER = None
    
def get_dbus_ifaces():
    return DBUS_MANAGER.get_dbus_ifaces()

def get_interface(iface):
    return DBUS_MANAGER.get_interface(iface)

def get_bus():
    return DBUS_MANAGER.get_bus()

def set_mainloop():
    return DBUS_MANAGER.set_mainloop()

def connect_to_dbus():
    return DBUS_MANAGER.connect_to_dbus()

def threads_init():
    dbus.mainloop.glib.threads_init()


class DBusManager(object):
    """ Manages the DBus objects used by wicd. """
    def __init__(self):
        self._bus = dbus.SystemBus()
        self._dbus_ifaces = {}  
    
    def get_dbus_ifaces(self):
        """ Returns a dict of dbus interfaces. """
        if not self._dbus_ifaces: connect_to_dbus()
        return self._dbus_ifaces
    
    def get_interface(self, iface):
        """ Returns a DBus Interface. """
        if not self._dbus_ifaces: connect_to_dbus()
        return self._dbus_ifaces[iface]
    
    def get_bus(self):
        """ Returns the loaded SystemBus. """
        return self._bus
    
    def set_mainloop(self, loop):
        dbus.set_default_main_loop(loop)
    
    def connect_to_dbus(self):
        """ Connects to wicd's dbus interfaces and loads them into a dict. """
        proxy_obj = self._bus.get_object("org.wicd.daemon", '/org/wicd/daemon')
        daemon = dbus.Interface(proxy_obj, 'org.wicd.daemon')
        
        proxy_obj = self._bus.get_object("org.wicd.daemon",
                                         '/org/wicd/daemon/wireless')
        wireless = dbus.Interface(proxy_obj, 'org.wicd.daemon.wireless')
        
        proxy_obj = self._bus.get_object("org.wicd.daemon",
                                         '/org/wicd/daemon/wired')
        wired = dbus.Interface(proxy_obj, 'org.wicd.daemon.wired')
        
        self._dbus_ifaces = {"daemon" : daemon, "wireless" : wireless,
                             "wired" : wired}
        
DBUS_MANAGER = DBusManager()


#!/usr/bin/python

""" prefs -- Wicd Preferences Dialog.

Displays the main settings dialog window for wicd and
handles recieving/sendings the settings from/to the daemon.

"""

#
#   Copyright (C) 2008-2009 Adam Blackburn
#   Copyright (C) 2008-2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import gobject
#import pango
import os

from wicd import misc
from wicd import wpath
from wicd import dbusmanager
from wicd.misc import checkboxTextboxToggle, noneToBlankString
from wicd.translations import _

daemon = None
wireless = None
wired = None

from wicd.translations import language

USER_SETTINGS_DIR = os.path.expanduser('~/.wicd/')

def setup_dbus():
    global daemon, wireless, wired
    daemon = dbusmanager.get_interface('daemon')
    wireless = dbusmanager.get_interface('wireless')
    wired = dbusmanager.get_interface('wired')

class PreferencesDialog(object):
    """ Class for handling the wicd preferences dialog window. """
    def __init__(self, parent, wTree):
        setup_dbus()
        self.parent = parent
        self.wTree = wTree
        self.prep_settings_diag()
        self.load_preferences_diag()
        
    def _setup_external_app_radios(self, radio_list, get_method, set_method):
        """ Generic function for setting up external app radios. """
        # Disable radios for apps that aren't installed.
        for app in radio_list[1:]:
            app.set_sensitive(daemon.GetAppAvailable(app.get_label()))
        selected_app = get_method()
        # Make sure the app we want to select is actually available.
        if radio_list[selected_app].get_property("sensitive"):
            radio_list[selected_app].set_active(True)
        else:
            # If it isn't, default to Automatic.
            set_method(misc.AUTO)
            radio_list[misc.AUTO].set_active(True)
            
    def load_preferences_diag(self):
        """ Loads data into the preferences Dialog. """
        
        self.wiredcheckbox.set_active(daemon.GetAlwaysShowWiredInterface())
        self.reconnectcheckbox.set_active(daemon.GetAutoReconnect())
        self.debugmodecheckbox.set_active(daemon.GetDebugMode())
        self.displaytypecheckbox.set_active(daemon.GetSignalDisplayType())
        self.verifyapcheckbox.set_active(daemon.GetShouldVerifyAp())
        self.preferwiredcheckbox.set_active(daemon.GetPreferWiredNetwork())
        self.showneverconnectcheckbox.set_active(daemon.GetShowNeverConnect())
        
        dhcp_list = [self.dhcpautoradio, self.dhclientradio, self.dhcpcdradio, 
                     self.pumpradio, self.udhcpcradio]
        self._setup_external_app_radios(dhcp_list, daemon.GetDHCPClient,
                                        daemon.SetDHCPClient)
        
        wired_link_list = [self.linkautoradio, self.ethtoolradio,
                           self.miitoolradio]
        self._setup_external_app_radios(wired_link_list,
                                        daemon.GetLinkDetectionTool,
                                        daemon.SetLinkDetectionTool)

        flush_list = [self.flushautoradio, self.ipflushradio,
                      self.routeflushradio]
        self._setup_external_app_radios(flush_list, daemon.GetFlushTool,
                                        daemon.SetFlushTool)
        
        sudo_list = [self.sudoautoradio, self.gksudoradio, self.kdesuradio,
                     self.ktsussradio]
        self._setup_external_app_radios(sudo_list, daemon.GetSudoApp,
                                        daemon.SetSudoApp)
        
        auto_conn_meth = daemon.GetWiredAutoConnectMethod()
        if auto_conn_meth == 1:
            self.usedefaultradiobutton.set_active(True)
        elif auto_conn_meth == 2:
            self.showlistradiobutton.set_active(True)
        elif auto_conn_meth == 3:
            self.lastusedradiobutton.set_active(True)
        
        self.entryWirelessInterface.set_text(daemon.GetWirelessInterface())
        self.entryWiredInterface.set_text(daemon.GetWiredInterface())

        def_driver = daemon.GetWPADriver()
        try:
            self.wpadrivercombo.set_active(self.wpadrivers.index(def_driver))
        except ValueError:
            self.wpadrivercombo.set_active(0)

        self.useGlobalDNSCheckbox.connect("toggled", checkboxTextboxToggle,
                                          (self.dns1Entry, self.dns2Entry,
                                           self.dns3Entry, self.dnsDomEntry, 
                                           self.searchDomEntry))

        dns_addresses = daemon.GetGlobalDNSAddresses()
        self.useGlobalDNSCheckbox.set_active(daemon.GetUseGlobalDNS())
        self.dns1Entry.set_text(noneToBlankString(dns_addresses[0]))
        self.dns2Entry.set_text(noneToBlankString(dns_addresses[1]))
        self.dns3Entry.set_text(noneToBlankString(dns_addresses[2]))
        self.dnsDomEntry.set_text(noneToBlankString(dns_addresses[3]))
        self.searchDomEntry.set_text(noneToBlankString(dns_addresses[4]))

        if not daemon.GetUseGlobalDNS():
            self.searchDomEntry.set_sensitive(False)
            self.dnsDomEntry.set_sensitive(False)
            self.dns1Entry.set_sensitive(False)
            self.dns2Entry.set_sensitive(False)
            self.dns3Entry.set_sensitive(False)
            
        cur_backend = daemon.GetSavedBackend()
        try:
            self.backendcombo.set_active(self.backends.index(cur_backend))
        except ValueError:
            self.backendcombo.set_active(0)

        self.notificationscheckbox.set_active(
                os.path.exists(
                    os.path.join(USER_SETTINGS_DIR, 'USE_NOTIFICATIONS')
                ))

        # if pynotify isn't installed disable the option
        try:
            import pynotify
        except ImportError:
            self.notificationscheckbox.set_active(False)
            self.notificationscheckbox.set_sensitive(False)

        # if notifications were disabled with the configure flag
        if wpath.no_use_notifications:
            self.notificationscheckbox.set_active(False)
            self.notificationscheckbox.hide()
            self.wTree.get_object('label2').hide()
        
        self.wTree.get_object("notebook2").set_current_page(0)
        
    def run(self):
        """ Runs the preferences dialog window. """
        return self.dialog.run()
    
    def hide(self):
        """ Hides the preferences dialog window. """
        self.dialog.hide()
        
    def destroy(self):
        self.dialog.destroy()
        
    def show_all(self):
        """ Shows the preferences dialog window. """
        self.dialog.show()
    
    def save_results(self):
        """ Pushes the selected settings to the daemon. """
        daemon.SetUseGlobalDNS(self.useGlobalDNSCheckbox.get_active())
        # Strip whitespace from DNS entries
        for i in [self.dns1Entry, self.dns2Entry, self.dns3Entry,
                  self.dnsDomEntry, self.searchDomEntry]:
            i.set_text(i.get_text().strip())
        daemon.SetGlobalDNS(self.dns1Entry.get_text(), self.dns2Entry.get_text(),
                            self.dns3Entry.get_text(), self.dnsDomEntry.get_text(),
                            self.searchDomEntry.get_text())
        daemon.SetWirelessInterface(self.entryWirelessInterface.get_text())
        daemon.SetWiredInterface(self.entryWiredInterface.get_text())
        daemon.SetWPADriver(self.wpadrivers[self.wpadrivercombo.get_active()])
        daemon.SetAlwaysShowWiredInterface(self.wiredcheckbox.get_active())
        daemon.SetAutoReconnect(self.reconnectcheckbox.get_active())
        daemon.SetDebugMode(self.debugmodecheckbox.get_active())
        daemon.SetSignalDisplayType(int(self.displaytypecheckbox.get_active()))
        daemon.SetShouldVerifyAp(bool(self.verifyapcheckbox.get_active()))
        daemon.SetPreferWiredNetwork(bool(self.preferwiredcheckbox.get_active()))
        daemon.SetShowNeverConnect(bool(self.showneverconnectcheckbox.get_active()))
        if self.showlistradiobutton.get_active():
            daemon.SetWiredAutoConnectMethod(2)
        elif self.lastusedradiobutton.get_active():
            daemon.SetWiredAutoConnectMethod(3)
        else:
            daemon.SetWiredAutoConnectMethod(1)

        daemon.SetBackend(self.backends[self.backendcombo.get_active()])
            
        # External Programs Tab
        if self.dhcpautoradio.get_active():
            dhcp_client = misc.AUTO
        elif self.dhclientradio.get_active():
            dhcp_client = misc.DHCLIENT
        elif self.dhcpcdradio.get_active():
            dhcp_client = misc.DHCPCD
        elif self.pumpradio.get_active():
            dhcp_client = misc.PUMP
        else:
            dhcp_client = misc.UDHCPC
        daemon.SetDHCPClient(dhcp_client)
        
        if self.linkautoradio.get_active():
            link_tool = misc.AUTO
        elif self.ethtoolradio.get_active():
            link_tool = misc.ETHTOOL
        else:
            link_tool = misc.MIITOOL
        daemon.SetLinkDetectionTool(link_tool)
        
        if self.flushautoradio.get_active():
            flush_tool = misc.AUTO
        elif self.ipflushradio.get_active():
            flush_tool = misc.IP
        else:
            flush_tool = misc.ROUTE
        daemon.SetFlushTool(flush_tool)
        
        if self.sudoautoradio.get_active():
            sudo_tool = misc.AUTO
        elif self.gksudoradio.get_active():
            sudo_tool = misc.GKSUDO
        elif self.kdesuradio.get_active():
            sudo_tool = misc.KDESU
        else:
            sudo_tool = misc.KTSUSS
        daemon.SetSudoApp(sudo_tool)

        [width, height] = self.dialog.get_size()
        
        not_path = os.path.join(USER_SETTINGS_DIR, 'USE_NOTIFICATIONS')
        if self.notificationscheckbox.get_active():
            if not os.path.exists(not_path):
                open(not_path, 'w')
        else:
            if os.path.exists(not_path):
                os.remove(not_path)
        # if this GUI was started by a tray icon,
        # instantly change the notifications there
        if self.parent.tray:
            self.parent.tray.icon_info.use_notify = \
                                self.notificationscheckbox.get_active()

    def set_label(self, glade_str, label):
        """ Sets the label for the given widget in wicd.glade. """
        self.wTree.get_object(glade_str).set_label(label)
        
    def prep_settings_diag(self):
        """ Set up anything that doesn't have to be persisted later. """
        def build_combobox(lbl):
            """ Sets up a ComboBox using the given widget name. """
            liststore = gtk.ListStore(gobject.TYPE_STRING)
            combobox = self.wTree.get_object(lbl)
            combobox.clear()
            combobox.set_model(liststore)
            cell = gtk.CellRendererText()
            combobox.pack_start(cell, True)
            combobox.add_attribute(cell, 'text', 0)
            return combobox
        
        def setup_label(name, lbl=""):
            """ Sets up a label for the given widget name. """
            widget = self.wTree.get_object(name)
            # if lbl:
            #     widget.set_label(lbl)
            if widget is None:
                raise ValueError('widget %s does not exist' % name)
            return widget
        
        # External Programs tab
        # self.wTree.get_object("gen_settings_label").set_label(_('General Settings'))
        # self.wTree.get_object("ext_prog_label").set_label(_('External Programs'))
        # self.wTree.get_object("dhcp_client_label").set_label(_('DHCP Client'))
        # self.wTree.get_object("wired_detect_label").set_label(_('Wired Link Detection'))
        # self.wTree.get_object("route_flush_label").set_label(_('Route Table Flushing'))
        # self.wTree.get_object("pref_backend_label").set_label(_('Backend') + ":")
        
        # entryWiredAutoMethod = self.wTree.get_object("pref_wired_auto_label")
        # entryWiredAutoMethod.set_label('Wired Autoconnect Setting:')
        # entryWiredAutoMethod.set_alignment(0, 0)
        # atrlist = pango.AttrList()
        # atrlist.insert(pango.AttrWeight(pango.WEIGHT_BOLD, 0, 50))
        # entryWiredAutoMethod.set_attributes(atrlist)
        
        # self.set_label("pref_dns1_label", "%s 1" % _('DNS server'))
        # self.set_label("pref_dns2_label", "%s 2" % _('DNS server'))
        # self.set_label("pref_dns3_label", "%s 3" % _('DNS server'))
        # self.set_label("pref_search_dom_label", "%s:" % _('Search domain'))
        # self.set_label("pref_wifi_label", "%s:" % _('Wireless Interface'))
        # self.set_label("pref_wired_label", "%s:" % _('Wired Interface'))
        # self.set_label("pref_driver_label", "%s:" % _('WPA Supplicant Driver'))
        
        self.dialog = self.wTree.get_object("pref_dialog")
        self.dialog.set_title(_('Preferences'))
        if os.path.exists(os.path.join(wpath.images, "wicd.png")):
            self.dialog.set_icon_from_file(os.path.join(wpath.images, "wicd.png"))
        width = int(gtk.gdk.screen_width() / 2.4)
        if width > 450:
            width = 450
        self.dialog.resize(width, int(gtk.gdk.screen_height() / 2))
            
        self.wiredcheckbox = setup_label("pref_always_check", _('''Always show wired interface'''))
        self.preferwiredcheckbox = setup_label("pref_prefer_wired_check",
                                               "prefer_wired")

        self.reconnectcheckbox = setup_label("pref_auto_check",
                                             _('Automatically reconnect on connection loss'))
        self.showneverconnectcheckbox = setup_label("pref_show_never_connect_check",
                                             _('Show never connect networks'))
        self.debugmodecheckbox = setup_label("pref_debug_check",
                                             _('Enable debug mode'))
        self.displaytypecheckbox = setup_label("pref_dbm_check",
                                               _('Use dBm to measure signal strength'))
        self.verifyapcheckbox = setup_label("pref_verify_ap_check",
                                            _('Ping static gateways after connecting to verify association'))
        self.usedefaultradiobutton = setup_label("pref_use_def_radio",
                                                 _('Use default profile on wired autoconnect'))
        self.showlistradiobutton = setup_label("pref_prompt_radio",
                                               _('Prompt for profile on wired autoconnect'))
        self.lastusedradiobutton = setup_label("pref_use_last_radio",
                                               _('Use last used profile on wired autoconnect'))

            
        self.notificationscheckbox = setup_label("pref_use_libnotify",
                                                 _('Display notifications about connection status'))

        # DHCP Clients
        self.dhcpautoradio = setup_label("dhcp_auto_radio", _('Automatic (recommended)'))
        self.dhclientradio = self.wTree.get_object("dhclient_radio")
        self.pumpradio = self.wTree.get_object("pump_radio")
        self.dhcpcdradio = self.wTree.get_object("dhcpcd_radio")
        self.udhcpcradio = self.wTree.get_object("udhcpc_radio")
        
        # Wired Link Detection Apps
        self.linkautoradio = setup_label("link_auto_radio", _('Automatic (recommended)'))
        self.linkautoradio = setup_label("link_auto_radio")
        self.ethtoolradio = setup_label("ethtool_radio")
        self.miitoolradio = setup_label("miitool_radio")
        
        # Route Flushing Apps
        self.flushautoradio = setup_label("flush_auto_radio",
                                          _('Automatic (recommended)'))
        self.ipflushradio = setup_label("ip_flush_radio")
        self.routeflushradio = setup_label("route_flush_radio")
        
        # Graphical Sudo Apps
        self.sudoautoradio = setup_label("sudo_auto_radio", _('Automatic (recommended)'))
        self.gksudoradio = setup_label("gksudo_radio")
        self.kdesuradio = setup_label("kdesu_radio")
        self.ktsussradio = setup_label("ktsuss_radio")

        # Replacement for the combo box hack
        self.wpadrivercombo = build_combobox("pref_wpa_combobox")
        self.wpadrivers = wireless.GetWpaSupplicantDrivers()
        self.wpadrivers.append("ralink_legacy")
        self.wpadrivers.append('none')

        for x in self.wpadrivers:
            self.wpadrivercombo.append_text(x)

        self.entryWirelessInterface = self.wTree.get_object("pref_wifi_entry")
        self.entryWiredInterface = self.wTree.get_object("pref_wired_entry")
        
        # Set up global DNS stuff
        self.useGlobalDNSCheckbox = setup_label("pref_global_check",
                                                'use_global_dns')
        self.searchDomEntry = self.wTree.get_object("pref_search_dom_entry")
        self.dnsDomEntry = self.wTree.get_object("pref_dns_dom_entry")
        self.dns1Entry = self.wTree.get_object("pref_dns1_entry")
        self.dns2Entry = self.wTree.get_object("pref_dns2_entry")
        self.dns3Entry = self.wTree.get_object("pref_dns3_entry")
        
        self.backendcombo = build_combobox("pref_backend_combobox")
        self.backendcombo.connect("changed", self.be_combo_changed)
        # Load backend combobox
        self.backends = daemon.GetBackendList()
        self.be_descriptions = daemon.GetBackendDescriptionDict()
        
        for x in self.backends:
            if x:
                if x == 'ioctl':
                    x = 'ioctl NOT SUPPORTED'
                self.backendcombo.append_text(x)
            
    def be_combo_changed(self, combo):
        """ Update the description label for the given backend. """
        self.backendcombo.set_tooltip_text(
            self.be_descriptions[self.backends[combo.get_active()]]
        )



#!/usr/bin/env python
"""This file is automatically generated by generate_version_info
It uses the current working tree to determine the revision.
So don't edit it. :)
"""

version_info = {'branch_nick': u'wicd',
 'build_date': '2012-04-30 21:26:29 +0200',
 'clean': None,
 'date': '2012-04-30 21:26:10 +0200',
 'revision_id': 'd.paleino@gmail.com-20120430192610-0eblawm4gmxvwn44',
 'revno': '768'}

revisions = {}

file_revisions = {}


if __name__ == '__main__':
    print 'revision: %(revno)s' % version_info
    print 'nick: %(branch_nick)s' % version_info
    print 'revision id: %(revision_id)s' % version_info



#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest
from wicd import misc

class TestMisc(unittest.TestCase):
    def test_misc_run(self):
        output = misc.Run(['echo', 'hi']).strip()
        self.assertEquals('hi', output)

    def test_valid_ip_1(self):
        self.assertTrue(misc.IsValidIP('0.0.0.0'))

    def test_valid_ip_2(self):
        self.assertTrue(misc.IsValidIP('255.255.255.255'))

    def test_valid_ip_3(self):
        self.assertTrue(misc.IsValidIP('10.0.1.1'))

    def test_invalid_ip_1(self):
        self.assertFalse(misc.IsValidIP('-10.0.-1.-1'))

    def test_invalid_ip_2(self):
        self.assertFalse(misc.IsValidIP('256.0.0.1'))

    def test_invalid_ip_3(self):
        self.assertFalse(misc.IsValidIP('1000.0.0.1'))

    def test_run_valid_regex(self):
        import re
        regex = re.compile('.*(ABC.EFG).*')
        found = misc.RunRegex(regex, '01234ABCDEFG56789')
        self.assertEquals(found, 'ABCDEFG')

    def test_run_invalid_regex(self):
        import re
        regex = re.compile('.*(ABC.EFG).*')
        found = misc.RunRegex(regex, '01234ABCEDFG56789')
        self.assertEquals(found, None)

    def test_to_boolean_false(self):
        self.assertFalse(misc.to_bool('False'))

    def test_to_boolean_0(self):
        self.assertFalse(misc.to_bool('0'))

    def test_to_boolean_true(self):
        self.assertTrue(misc.to_bool('True'))

    def test_to_boolean_true(self):
        self.assertTrue(misc.to_bool('1'))

    def test_noneify_1(self):
        self.assertEquals(misc.Noneify('None'), None)

    def test_noneify_2(self):
        self.assertEquals(misc.Noneify(''), None)

    def test_noneify_3(self):
        self.assertEquals(misc.Noneify(None), None)

    def test_noneify_4(self):
        self.assertFalse(misc.Noneify('False'))

    def test_noneify_5(self):
        self.assertFalse(misc.Noneify('0'))

    def test_noneify_6(self):
        self.assertFalse(misc.Noneify(False))

    def test_noneify_7(self):
        self.assertTrue(misc.Noneify('True'))

    def test_noneify_8(self):
        self.assertTrue(misc.Noneify('1'))

    def test_noneify_9(self):
        self.assertTrue(misc.Noneify(True))

    def test_noneify_10(self):
        self.assertEquals(misc.Noneify('randomtext'), 'randomtext')

    def test_noneify_11(self):
        self.assertEquals(misc.Noneify(5), 5)

    def test_none_to_string_1(self):
        self.assertEquals(misc.noneToString(None), 'None')

    def test_none_to_string_2(self):
        self.assertEquals(misc.noneToString(''), 'None')

    def test_none_to_string_3(self):
        self.assertEquals(misc.noneToString(None), 'None')

    ####################################################################
    # misc.to_unicode actually converts to utf-8, which is type str    #
    ####################################################################

    def test_to_unicode_1(self):
        self.assertEquals(misc.to_unicode('邪悪'), '邪悪')

    def test_to_unicode_2(self):
        self.assertEquals(misc.to_unicode(u'邪悪'), '邪悪')

    def test_to_unicode_3(self):
        self.assertEquals(misc.to_unicode(u'abcdef'), 'abcdef')

    def test_to_unicode_4(self):
        self.assertEquals(type(misc.to_unicode('abcdef'.encode('latin-1'))), str)

    def test_to_unicode_5(self):
        self.assertEquals(misc.to_unicode("berkåk"), "berkåk")

    def test_to_unicode_6(self):
        self.assertEquals(misc.to_unicode('berk\xe5k'), "berkåk")

    def test_none_to_blank_string_1(self):
        self.assertEquals(misc.noneToBlankString(None), '')

    def test_none_to_blank_string_2(self):
        self.assertEquals(misc.noneToBlankString('None'), '')

    def test_string_to_none_1(self):
        self.assertEquals(misc.stringToNone(''), None)

    def test_string_to_none_2(self):
        self.assertEquals(misc.stringToNone('None'), None)

    def test_string_to_none_3(self):
        self.assertEquals(misc.stringToNone(None), None)

    def test_string_to_none_4(self):
        self.assertEquals(misc.stringToNone('abcdef'), 'abcdef')

def suite():
	suite = unittest.TestSuite()
	tests = []
	[ tests.append(test) for test in dir(TestMisc) if test.startswith('test') ]
	for test in tests:
		suite.addTest(TestMisc(test))
	return suite

if __name__ == '__main__':
	unittest.main()



#!/usr/bin/env python
#
#   Copyright (C) 2007 - 2009 Adam Blackburn
#   Copyright (C) 2007 - 2009 Dan O'Reilly
#   Copyright (C) 2009        Andrew Psaltis
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from distutils.core import setup, Command
from distutils.extension import Extension
import os
import sys
import shutil
import subprocess
from glob import glob

# Be sure to keep this updated!
# VERSIONNUMBER
VERSION_NUM = '1.7.2.4'
# REVISION_NUM is automatically updated
REVISION_NUM = 'unknown'
CURSES_REVNO = 'uimod'

# change to the directory setup.py is contained in
os.chdir(os.path.abspath(os.path.split(__file__)[0]))

try:
    if os.path.exists('.bzr') and os.system('bzr > /dev/null 2>&1') == 0:
        try:
            os.system('bzr version-info --python > vcsinfo.py')
        except:
            pass
    import vcsinfo
    REVISION_NUM = vcsinfo.version_info['revno']
except Exception, e:
    print 'failed to find revision number:'
    print e

class configure(Command):
    description = "configure the paths that Wicd will be installed to"
    
    user_options = [
        # The first bunch is DIRECTORIES - they need to end with a slash ("/"),
        # which will automatically be tacked on in the finalize_options method
        ('lib=', None, 'set the lib directory'),
        ('share=', None, 'set the share directory'),
        ('etc=', None, 'set the etc directory'),
        ('scripts=', None, 'set the global scripts directory'),
        ('pixmaps=', None, 'set the pixmaps directory'),
        ('images=', None, 'set the image directory'),
        ('encryption=', None, 'set the encryption template directory'),
        ('bin=', None, 'set the bin directory'),
        ('sbin=', None, 'set the sbin directory'),
        ('backends=', None, 'set the backend storage directory'),
        ('daemon=', None, 'set the daemon directory'),
        ('curses=', None, 'set the curses UI directory'),
        ('gtk=', None, 'set the GTK UI directory'),
        ('cli=', None, 'set the CLI directory'),
        ('networks=', None, 'set the encryption configuration directory'),
        ('log=', None, 'set the log directory'),
        ('resume=', None, 'set the directory the resume from suspend script is stored in'),
        ('suspend=', None, 'set the directory the  suspend script is stored in'),
        ('pmutils=', None, 'set the directory the  pm-utils hooks are stored in'),
        ('dbus=', None, 'set the directory the dbus config file is stored in'),
        ('dbus-service=', None, 'set the directory where the dbus services config files are stored in'),
        ('systemd=', None, 'set the directory where the systemd system services config files are stored in'),
        ('logrotate=', None, 'set the directory where the logrotate configuration files are stored in'),
        ('desktop=', None, 'set the directory the .desktop file is stored in'),
        ('icons=', None, "set the base directory for the .desktop file's icons"),
        ('translations=', None, 'set the directory translations are stored in'),
        ('autostart=', None, 'set the directory that will be autostarted on desktop login'),
        ('varlib=',None , 'set the path for wicd\'s variable state data'),
        ('init=', None, 'set the directory for the init file'),
        ('docdir=', None, 'set the directory for the documentation'),
        ('mandir=', None, 'set the directory for the man pages'),
        ('kdedir=', None, 'set the kde autostart directory'),
        
        # Anything after this is a FILE; in other words, a slash ("/") will 
        # not automatically be added to the end of the path.
        # Do NOT remove the python= entry, as it signals the beginning of 
        # the file section.
        ('python=', None, 'set the path to the Python executable'),
        ('pidfile=', None, 'set the pid file'),
        ('initfile=', None, 'set the init file to use'),
        ('initfilename=', None, "set the name of the init file (don't use)"),
        ('wicdgroup=', None, "set the name of the group used for wicd"),
        ('distro=', None, 'set the distribution for which wicd will be installed'),
        ('loggroup=', None, 'the group the log file belongs to'),
        ('logperms=', None, 'the log file permissions'),

        # Configure switches
        ('no-install-init', None, "do not install the init file"),
        ('no-install-man', None, 'do not install the man files'),
        ('no-install-i18n-man', None, 'do not install the translated man files'),
        ('no-install-kde', None, 'do not install the kde autostart file'),
        ('no-install-acpi', None, 'do not install the suspend.d and resume.d acpi scripts'),
        ('no-install-pmutils', None, 'do not install the pm-utils hooks'),
        ('no-install-docs', None, 'do not install the auxiliary documentation'),
        ('no-install-ncurses', None, 'do not install the ncurses client'),
        ('no-install-cli', None, 'do not install the command line executable'),
        ('no-install-gtk', None, 'do not install the gtk client'),
        ('no-use-notifications', None, 'do not ever allow the use of libnotify notifications')
        ]
        
    def initialize_options(self):
        self.lib = '/usr/lib/wicd/'
        self.share = '/usr/share/wicd/'
        self.etc = '/etc/wicd/'
        self.scripts = self.etc + "scripts/"
        self.icons = '/usr/share/icons/hicolor/'
        self.pixmaps = '/usr/share/pixmaps/'
        self.images = self.pixmaps + 'wicd/'
        self.encryption = self.etc + 'encryption/templates/'
        self.bin = '/usr/bin/'
        self.sbin = '/usr/sbin/'
        self.daemon = self.share + 'daemon'
        self.backends = self.share + 'backends'
        self.curses = self.share + 'curses'
        self.gtk = self.share + 'gtk'
        self.cli = self.share + 'cli'
        self.varlib = '/var/lib/wicd/'
        self.networks = self.varlib + 'configurations/'
        self.log = '/var/log/wicd/'
        self.resume = '/etc/acpi/resume.d/'
        self.suspend = '/etc/acpi/suspend.d/'
        self.pmutils = '/usr/lib/pm-utils/sleep.d/'
        self.dbus = '/etc/dbus-1/system.d/'
        self.dbus_service = '/usr/share/dbus-1/system-services/'
        self.systemd = '/lib/systemd/system/'
        self.logrotate = '/etc/logrotate.d/'
        self.desktop = '/usr/share/applications/'
        self.translations = '/usr/share/locale/'
        self.autostart = '/etc/xdg/autostart/'
        self.docdir = '/usr/share/doc/wicd/'
        self.mandir = '/usr/share/man/'
        self.kdedir = '/usr/share/autostart/'
        self.distro = 'auto'
        
        self.no_install_init = False
        self.no_install_man = False
        self.no_install_i18n_man = False
        self.no_install_kde = False
        self.no_install_acpi = False
        self.no_install_pmutils = False
        self.no_install_docs = False
        self.no_install_gtk = False
        self.no_install_ncurses = False
        self.no_install_cli = False
        self.no_use_notifications = False

        # Determine the default init file location on several different distros
        self.distro_detect_failed = False
        
        self.initfile = 'init/default/wicd'
        # ddistro is the detected distro
        if os.path.exists('/etc/redhat-release'):
            self.ddistro = 'redhat'
        elif os.path.exists('/etc/SuSE-release'):
            self.ddistro = 'suse'
        elif os.path.exists('/etc/fedora-release'):
            self.ddistro = 'redhat'
        elif os.path.exists('/etc/gentoo-release'):
            self.ddistro = 'gentoo'
        elif os.path.exists('/etc/debian_version'):
            self.ddistro = 'debian'
        elif os.path.exists('/etc/arch-release'):
            self.ddistro = 'arch'
        elif os.path.exists('/etc/slackware-version') or \
             os.path.exists('/etc/slamd64-version') or \
             os.path.exists('/etc/bluewhite64-version'):
            self.ddistro = 'slackware'
        elif os.path.exists('/etc/pld-release'):
            self.ddistro = 'pld'
        elif os.path.exists('/usr/bin/crux'):
            self.ddistro = 'crux'
        elif os.path.exists('/etc/lunar.release'):
            self.distro = 'lunar'
        else:
            self.ddistro = 'FAIL'
            #self.no_install_init = True
            #self.distro_detect_failed = True
            print 'WARNING: Unable to detect the distribution in use.  ' + \
                  'If you have specified --distro or --init and --initfile, configure will continue.  ' + \
                  'Please report this warning, along with the name of your ' + \
                  'distribution, to the wicd developers.'

        # Try to get the pm-utils sleep hooks directory from pkg-config and
        # the kde prefix from kde-config
        # Don't run these in a shell because it's not needed and because shell 
        # swallows the OSError we would get if {pkg,kde}-config do not exist
        # If we don't get anything from *-config, or it didn't run properly, 
        # or the path is not a proper absolute path, raise an error
        try:
            pmtemp = subprocess.Popen(["pkg-config", "--variable=pm_sleephooks", 
                                       "pm-utils"], stdout=subprocess.PIPE)
            returncode = pmtemp.wait() # let it finish, and get the exit code
            pmutils_candidate = pmtemp.stdout.readline().strip() # read stdout
            if len(pmutils_candidate) == 0 or returncode != 0 or \
               not os.path.isabs(pmutils_candidate):
                raise ValueError
            else:
                self.pmutils = pmutils_candidate
        except (OSError, ValueError):
            pass # use our default

        try:
            kdetemp = subprocess.Popen(["kde-config","--prefix"], stdout=subprocess.PIPE)
            returncode = kdetemp.wait() # let it finish, and get the exit code
            kdedir_candidate = kdetemp.stdout.readline().strip() # read stdout
            if len(kdedir_candidate) == 0 or returncode != 0 or \
               not os.path.isabs(kdedir_candidate):
                raise ValueError
            else:
                self.kdedir = kdedir_candidate + '/share/autostart'
        except (OSError, ValueError):
            # If kde-config isn't present, we'll check for kde-4.x
            try:
                kde4temp = subprocess.Popen(["kde4-config","--prefix"], stdout=subprocess.PIPE)
                returncode = kde4temp.wait() # let it finish, and get the exit code
                kde4dir_candidate = kde4temp.stdout.readline().strip() # read stdout
                if len(kde4dir_candidate) == 0 or returncode != 0 or \
                   not os.path.isabs(kde4dir_candidate):
                    raise ValueError
                else:
                    self.kdedir = kde4dir_candidate + '/share/autostart'
            except (OSError, ValueError):
                # If neither kde-config nor kde4-config are not present or 
                # return an error, then we can assume that kde isn't installed
                # on the user's system
                self.no_install_kde = True
                # If the assumption above turns out to be wrong, do this:
                #pass # use our default

        self.python = '/usr/bin/python'
        self.pidfile = '/var/run/wicd/wicd.pid'
        self.initfilename = os.path.basename(self.initfile)
        self.wicdgroup = 'users'
        self.loggroup = ''
        self.logperms = '0600'

    def distro_check(self):
        print "Distro is: " + self.distro
        if self.distro in ['sles', 'suse']:
            self.init = '/etc/init.d/'
            self.initfile = 'init/suse/wicd'
        elif self.distro in ['redhat', 'centos', 'fedora']:
            self.init = '/etc/rc.d/init.d/'
            self.initfile = 'init/redhat/wicd'
            self.pidfile = '/var/run/wicd.pid'
        elif self.distro in ['slackware', 'slamd64', 'bluewhite64']:
            self.init = '/etc/rc.d/'
            self.initfile = 'init/slackware/rc.wicd'
            self.docdir = '/usr/doc/wicd-%s' % VERSION_NUM
            self.mandir = '/usr/man/'
            self.no_install_acpi = True
            self.wicdgroup = "netdev"
        elif self.distro in ['debian']:
            self.wicdgroup = "netdev"
            self.loggroup = "adm"
            self.logperms = "0640"
            self.init = '/etc/init.d/'
            self.initfile = 'init/debian/wicd'
        elif self.distro in ['arch']:
            self.init = '/etc/rc.d/'
            self.initfile = 'init/arch/wicd'
        elif self.distro in ['gentoo']:
            self.init = '/etc/init.d/'
            self.initfile = 'init/gentoo/wicd'
        elif self.distro in ['pld']:
            self.init = '/etc/rc.d/init.d/'
            self.initfile = 'init/pld/wicd'
        elif self.distro in ['crux']:
            self.init = '/etc/rc.d/'
        elif self.distro in ['lunar']:
            self.init='/etc/init.d/'
            self.initfile = 'init/lunar/wicd'
        else :
            if self.distro == 'auto':
                print "NOTICE: Automatic distro detection found: " + self.ddistro + ", retrying with that..."
                self.distro = self.ddistro
                self.distro_check()
            else:
                print "WARNING: Distro detection failed!"
                self.init='init/default/wicd'
                self.no_install_init = True
                self.distro_detect_failed = True
        


    def finalize_options(self):
        self.distro_check()
        if self.distro_detect_failed and not self.no_install_init and \
           'FAIL' in [self.init, self.initfile]:
            print 'ERROR: Failed to detect distro. Configure cannot continue.  ' + \
                  'Please specify --init and --initfile to continue with configuration.'

        # loop through the argument definitions in user_options
        for argument in self.user_options:
            # argument name is the first item in the user_options list
            # sans the = sign at the end
            argument_name = argument[0][:-1]
            # select the first one, which is the name of the option
            value = getattr(self, argument_name.replace('-', '_'))
            # if the option is not python (which is not a directory)
            if not argument[0][:-1] == "python":
                # see if it ends with a /
                if not value.endswith("/"):
                    # if it doesn't, slap one on
                    setattr(self, argument_name, value + "/")
            else:
                # as stated above, the python entry defines the beginning
                # of the files section
                return

    def run(self):
        values = list()
        for argument in self.user_options:
            if argument[0].endswith('='):
                cur_arg = argument[0][:-1]
                cur_arg_value = getattr(self, cur_arg.replace('-', '_'))
                print "%s is %s" % (cur_arg, cur_arg_value)
                values.append((cur_arg, getattr(self, cur_arg.replace('-','_'))))
            else:
                cur_arg = argument[0]
                cur_arg_value = getattr(self, cur_arg.replace('-', '_'))
                print "Found switch %s %s" % (argument, cur_arg_value) 
                values.append((cur_arg, bool(cur_arg_value)))
        
        print 'Replacing values in template files...'
        for item in os.listdir('in'):
            if item.endswith('.in'):
                print 'Replacing values in',item,
                original_name = os.path.join('in',item)
                item_in = open(original_name, 'r')
                final_name = item[:-3].replace('=','/')
                parent_dir = os.path.dirname(final_name)
                if parent_dir and not os.path.exists(parent_dir):
                    print '(mkdir %s)'%parent_dir,
                    os.makedirs(parent_dir)
                print final_name
                item_out = open(final_name, 'w')
                for line in item_in.readlines():
                    for item, value in values:
                        line = line.replace('%' + str(item.upper().replace('-','_')) + \
                                            '%', str(value))

                    # other things to replace that aren't arguments
                    line = line.replace('%VERSION%', str(VERSION_NUM))
                    line = line.replace('%REVNO%', str(REVISION_NUM))
                    line = line.replace('%CURSES_REVNO%', str(CURSES_REVNO))
                        
                    item_out.write(line)
                
                item_out.close()
                item_in.close()
                shutil.copymode(original_name, final_name)

        trans = compile_translations(self.distribution)
        trans.run()

class clear_generated(Command):
    description = 'clears out files generated by configure'

    user_options = []

    def initialize_options(self):
        pass
        
    def finalize_options(self):
        pass
        
    def run(self):
        print 'Removing completed template files...'
        for item in os.listdir('in'):
            if item.endswith('.in'):
                print 'Removing completed',item,
                original_name = os.path.join('in',item)
                final_name = item[:-3].replace('=','/')
                print final_name, '...',
                if os.path.exists(final_name):
                    os.remove(final_name)
                    print 'Removed.'
                else:
                    print 'Does not exist.'
        print 'Removing compiled translation files...'
        if os.path.exists('translations'):
            shutil.rmtree('translations/')
        os.makedirs('translations/')

class test(Command):
    description = "run Wicd's unit tests"
	
    user_options = []
	
    def initialize_options(self):
        pass
        
    def finalize_options(self):
        pass
        
    def run(self):
        print "importing tests"
        import tests
        print 'running tests'
        tests.run_tests()

class update_message_catalog(Command):
    description = "update wicd.pot with new strings"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('pybabel extract . -o po/wicd.pot --sort-output')
        os.system('xgettext -L glade data/wicd.ui -j -o po/wicd.pot')
        
class update_translations(Command):
    description = "update po-files with new strings from wicd.pot"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        for pofile in glob('po/*.po'):
            lang = pofile.replace('po/', '').replace('.po', '')
            os.system('pybabel update -o %s -i po/wicd.pot -D wicd -l %s' % (pofile, lang))

class compile_translations(Command):
    description = 'compile po-files to binary mo'

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if os.path.exists('translations'):
            shutil.rmtree('translations/')
        os.makedirs('translations')
        for pofile in glob('po/*.po'):
            lang = pofile.replace('po/', '').replace('.po', '')
            os.makedirs('translations/' + lang + '/LC_MESSAGES/')
            os.system('pybabel compile -D wicd -i %s -l %s -d translations/' % (pofile, lang))
        

class uninstall(Command):
    description = "remove Wicd using uninstall.sh and install.log"
         
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system("./uninstall.sh")

try:
    import wpath
except ImportError:
    print '''Error importing wpath.py. You can safely ignore this
message. It is probably because you haven't run python setup.py
configure yet or you are running it for the first time.'''

data = []
py_modules = ['wicd.networking','wicd.misc','wicd.wnettools',
              'wicd.wpath','wicd.dbusmanager',
              'wicd.logfile','wicd.backend','wicd.configmanager',
              'wicd.translations']

# path to the file to put in empty directories
# fixes https://bugs.launchpad.net/wicd/+bug/503028
empty_file = 'other/.empty_on_purpose'

try:
    print "Using init file",(wpath.init, wpath.initfile)
    data = [
        (wpath.dbus, ['other/wicd.conf']),
        (wpath.dbus_service, ['other/org.wicd.daemon.service']),
        (wpath.systemd, ['other/wicd.service']),
        (wpath.logrotate, ['other/wicd.logrotate']),
        (wpath.log, [empty_file]), 
        (wpath.etc, ['other/dhclient.conf.template.default']),
        (wpath.encryption, [('encryption/templates/' + b) for b in 
                            os.listdir('encryption/templates') if not b.startswith('.')]),
        (wpath.networks, [empty_file]),
        (wpath.sbin,  ['scripts/wicd']),  
        (wpath.daemon, ['wicd/monitor.py', 'wicd/wicd-daemon.py',
                    'wicd/suspend.py', 'wicd/autoconnect.py']), 
        (wpath.backends, ['wicd/backends/be-external.py', 'wicd/backends/be-ioctl.py']),
        (wpath.scripts, [empty_file]),
        (wpath.predisconnectscripts, [empty_file]),
        (wpath.postdisconnectscripts, [empty_file]),
        (wpath.preconnectscripts, [empty_file]),
        (wpath.postconnectscripts, [empty_file]),
    ]

    if not wpath.no_install_gtk:
        data.append((wpath.desktop, ['other/wicd.desktop']))
        data.append((wpath.bin, ['scripts/wicd-client']))
        data.append((wpath.bin, ['scripts/wicd-gtk']))
        data.append((wpath.gtk, [
                                 'gtk/wicd-client.py',
                                 'gtk/netentry.py',
                                 'gtk/prefs.py',
                                 'gtk/gui.py',
                                 'gtk/guiutil.py',
                                 'data/wicd.ui',
                                 'gtk/configscript.py',
                                ]))
        data.append((wpath.autostart, ['other/wicd-tray.desktop']))
        if not wpath.no_install_man:
            data.append((wpath.mandir + 'man1/', [ 'man/wicd-client.1' ]))
        data.append((wpath.icons + 'scalable/apps/', ['icons/scalable/wicd-gtk.svg']))
        data.append((wpath.icons + '192x192/apps/', ['icons/192px/wicd-gtk.png']))
        data.append((wpath.icons + '128x128/apps/', ['icons/128px/wicd-gtk.png']))
        data.append((wpath.icons + '96x96/apps/', ['icons/96px/wicd-gtk.png']))
        data.append((wpath.icons + '72x72/apps/', ['icons/72px/wicd-gtk.png']))
        data.append((wpath.icons + '64x64/apps/', ['icons/64px/wicd-gtk.png']))
        data.append((wpath.icons + '48x48/apps/', ['icons/48px/wicd-gtk.png']))
        data.append((wpath.icons + '36x36/apps/', ['icons/36px/wicd-gtk.png']))
        data.append((wpath.icons + '32x32/apps/', ['icons/32px/wicd-gtk.png']))
        data.append((wpath.icons + '24x24/apps/', ['icons/24px/wicd-gtk.png']))
        data.append((wpath.icons + '22x22/apps/', ['icons/22px/wicd-gtk.png']))
        data.append((wpath.icons + '16x16/apps/', ['icons/16px/wicd-gtk.png']))
        data.append((wpath.images, [('images/' + b) for b in os.listdir('images') if not b.startswith('.')]))
        data.append((wpath.pixmaps, ['other/wicd-gtk.xpm']))
    if not wpath.no_install_ncurses:
        data.append((wpath.curses, ['curses/curses_misc.py']))
        data.append((wpath.curses, ['curses/prefs_curses.py']))
        data.append((wpath.curses, ['curses/wicd-curses.py']))
        data.append((wpath.curses, ['curses/netentry_curses.py']))
        data.append((wpath.curses, ['curses/configscript_curses.py']))
        data.append((wpath.bin, ['scripts/wicd-curses'])) 
        if not wpath.no_install_man:
            data.append(( wpath.mandir + 'man8/', ['man/wicd-curses.8'])) 
        if not wpath.no_install_man and not wpath.no_install_i18n_man:
            data.append(( wpath.mandir + 'nl/man8/', ['man/nl/wicd-curses.8'])) 
        if not wpath.no_install_docs:
            data.append(( wpath.docdir, ['curses/README.curses'])) 
    if not wpath.no_install_cli:
        data.append((wpath.cli, ['cli/wicd-cli.py']))
        data.append((wpath.bin, ['scripts/wicd-cli'])) 
        if not wpath.no_install_man:
            data.append(( wpath.mandir + 'man8/', ['man/wicd-cli.8'])) 
        if not wpath.no_install_docs:
            data.append(( wpath.docdir, ['cli/README.cli'])) 
    piddir = os.path.dirname(wpath.pidfile)
    if not piddir.endswith('/'):
        piddir += '/'
    if not wpath.no_install_docs:
        data.append((wpath.docdir, ['INSTALL', 'LICENSE', 'AUTHORS',
                                     'README', 'CHANGES', ]))
        data.append((wpath.varlib, ['other/WHEREAREMYFILES']))
    if not wpath.no_install_kde:
        if not wpath.no_install_gtk:
            data.append((wpath.kdedir, ['other/wicd-tray.desktop']))
    if not wpath.no_install_init:
        data.append((wpath.init, [ wpath.initfile ]))
    if not wpath.no_install_man:
        data.append((wpath.mandir + 'man8/', ['man/wicd.8']))
        data.append((wpath.mandir + 'man5/', ['man/wicd-manager-settings.conf.5']))
        data.append((wpath.mandir + 'man5/', ['man/wicd-wired-settings.conf.5']))
        data.append((wpath.mandir + 'man5/', ['man/wicd-wireless-settings.conf.5']))
        data.append((wpath.mandir + 'man1/', ['man/wicd-client.1']))
    if not wpath.no_install_man and not wpath.no_install_i18n_man:
        # Dutch translations of the man
        data.append((wpath.mandir + 'nl/man8/', ['man/nl/wicd.8' ]))
        data.append((wpath.mandir + 'nl/man5/', ['man/nl/wicd-manager-settings.conf.5']))
        data.append((wpath.mandir + 'nl/man5/', ['man/nl/wicd-wired-settings.conf.5']))
        data.append((wpath.mandir + 'nl/man5/', ['man/nl/wicd-wireless-settings.conf.5']))
        data.append((wpath.mandir + 'nl/man1/', ['man/nl/wicd-client.1']))
    if not wpath.no_install_acpi:
        data.append((wpath.resume, ['other/80-wicd-connect.sh']))
        data.append((wpath.suspend, ['other/50-wicd-suspend.sh']))
    if not wpath.no_install_pmutils:
        data.append((wpath.pmutils, ['other/55wicd']))
    print 'Using pid path', os.path.basename(wpath.pidfile)
    print 'Language support for',
    for pofile in glob('po/*.po'):
        language = pofile.replace('po/', '').replace('.po', '')
        print language,
        data.append((wpath.translations + language + '/LC_MESSAGES/',
                    ['translations/' + language + '/LC_MESSAGES/wicd.mo']))
    print
except Exception, e:
    print str(e)
    print '''Error setting up data array. This is normal if 
python setup.py configure has not yet been run.'''


wpactrl_ext = Extension(name = 'wpactrl', 
                        sources = ['depends/python-wpactrl/wpa_ctrl.c',
                                   'depends/python-wpactrl/wpactrl.c'],
                        extra_compile_args = ["-fno-strict-aliasing"])

iwscan_ext = Extension(name = 'iwscan', libraries = ['iw'],
                       sources = ['depends/python-iwscan/pyiwscan.c'])
    
setup(
    cmdclass = {
        'configure' : configure,
        'uninstall' : uninstall,
        'test' : test,
        'clear_generated' : clear_generated,
        'update_message_catalog' : update_message_catalog,
        'update_translations' : update_translations,
        'compile_translations' : compile_translations,
    },
    name = "wicd",
    version = VERSION_NUM,
    description = "A wireless and wired network manager",
    long_description = """A complete network connection manager
Wicd supports wired and wireless networks, and capable of
creating and tracking profiles for both.  It has a 
template-based wireless encryption system, which allows the user
to easily add encryption methods used.  It ships with some common
encryption types, such as WPA and WEP. Wicd will automatically
connect at startup to any preferred network within range.
""",
    author = "Adam Blackburn, Dan O'Reilly, Andrew Psaltis, David Paleino",
    author_email = "compwiz18@gmail.com, oreilldf@gmail.com, ampsaltis@gmail.com, d.paleino@gmail.com",
    url = "https://launchpad.net/wicd",
    license = "http://www.gnu.org/licenses/old-licenses/gpl-2.0.html",
    py_modules = py_modules,
    data_files = data,
)
